from ufs_tools.app_tools import get_executable_folder

from ufs_tools import get_folder, os
from ufs_tools.inspect_utils import get_parent_frame_file, get_inspection_frame
from ufs_tools.libtool import include_all_direct_subfolders

from djangoautoconf import DjangoAutoConf


class DjangoDevServerAutoConf(object):
    def __init__(self):
        self.external_app_repositories = "external_app_repos"
        if "ROOT_DIR" in os.environ:
            self.root_dir = os.environ["ROOT_DIR"]
        else:
            self.root_dir = get_executable_folder()
        self.server_base_package_folder_name = "server_base_packages"
        self.django_auto_conf = DjangoAutoConf()

    def set_root_dir(self, root_dir):
        self.root_dir = root_dir

    def set_external_app_repositories(self, external_app_repositories):
        self.external_app_repositories = external_app_repositories

    def configure(self, extra_settings_folder=None):
        self.django_auto_conf.set_root_dir(self.root_dir)
        self.django_auto_conf.set_external_app_repositories(self.external_app_repositories)
        if not (extra_settings_folder is None):
            self.django_auto_conf.local_app_setting_folders.append(extra_settings_folder)
        self.django_auto_conf.configure()


if __name__ == "__main__":
    d = DjangoDevServerAutoConf()
    print(d)





LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
        },
        'simple': {
            'format': '%(levelname)s %(asctime)s %(message)s'
        },
    },
    'handlers': {
        'db_log': {
            'level': 'DEBUG',
            'class': 'django_db_logger.db_log_handler.DatabaseLogHandler'
        },
    },
    'loggers': {
        'db': {
            'handlers': ['db_log'],
            'level': 'DEBUG'
        }
    }
}



from djangoautoconf.auto_conf_admin_tools.admin_feature_base import AdminFeatureBase


# Ref: http://stackoverflow.com/questions/8992865/django-admin-sort-foreign-key-field-list
def formfield_for_foreignkey(self, db_field, request, **kwargs):
    if db_field.name == self.admin_feature_foreign_key_sort.sort_attribute:
        kwargs["queryset"] = db_field.related_model.objects.order_by(
            self.admin_feature_foreign_key_sort.sort_field_of_foreign_key)
    return super(self.__class__, self).formfield_for_foreignkey(db_field, request, **kwargs)


class ForeignKeySortFeature(AdminFeatureBase):

    def __init__(self):
        super(ForeignKeySortFeature, self).__init__()
        self.model_class = None
        self.sort_attribute = None
        self.sort_field_of_foreign_key = None

    def process_admin_class_attr(self, admin_attr, class_inst):
        # super(ForeignKeySortFeature, self).process_admin_class_attr(admin_attr, class_inst)
        self.model_class = class_inst
        admin_attr["admin_feature_foreign_key_sort"] = self
        admin_attr["formfield_for_foreignkey"] = formfield_for_foreignkey




from djangoautoconf.auto_conf_admin_tools.admin_feature_base import AdminFeatureBase
from django.conf import settings

from djangoautoconf.model_utils.import_export_utils import get_import_export_admin_mixin

__author__ = 'weijia'


class ImportExportFeature(AdminFeatureBase):

    def __init__(self):
        super(ImportExportFeature, self).__init__()
        self.is_import_export_supported = False

        if "import_export" in settings.INSTALLED_APPS:
            self.is_import_export_supported = True

    def process_parent_class_list(self, parent_list, class_inst):
        if self.is_import_export_supported:
            try:
                # from import_export.admin import ImportExportActionModelAdmin
                parent_list.append(get_import_export_admin_mixin(class_inst))
            except ImportError:
                pass

    def process_admin_class_attr(self, admin_attr, class_inst):
        # if self.is_import_export_supported:
        #     admin_attr.update({
        #         "resource_class": get_import_export_resource(class_inst)
        #     })
        pass




from djangoautoconf.auto_conf_admin_tools.admin_feature_base import AdminFeatureBase

__author__ = 'weijia'


class AdminAttrFeature(AdminFeatureBase):

    def __init__(self, field_value_dict):
        """
        Init the fields and values
        :param field_value_dict: example: {"filter_horizontal": ("descriptions", )}
        """
        super(AdminAttrFeature, self).__init__()
        self.admin_attributes = field_value_dict

    def process_admin_class_attr(self, admin_attr, class_inst):
        admin_attr.update(self.admin_attributes)



import sys

__author__ = 'weijia'
import django.dispatch


before_server_start = django.dispatch.Signal(providing_args=[])
before_server_stop = django.dispatch.Signal(providing_args=[])


class ServerSignalTrigger(object):
    def trigger_server_start_if_needed(self):
        if sys.argv[1] == "runserver":
            before_server_start.send(sender=self)

    def trigger_server_stop_if_needed(self):
        if sys.argv[1] == "runserver":
            before_server_stop.send(sender=self)
        print "Process exiting"


import importlib
import logging
import os

from django.utils.crypto import get_random_string
from .auto_conf_utils import enum_modules, enum_folders
from ufs_tools.basic_lib_tool import remove_folder_in_sys_path, include
from ufs_tools.folder_tool import ensure_dir

from .base_setting_storage import ObjectSettingStorage

__author__ = 'weijia'

log = logging.getLogger(__name__)


def get_feature_setting_module_list(features):
    ordered_import_list = [
        # self.default_settings_import_str,
        "djangoautoconf.features.djangoautoconf_settings",
        # "djangoautoconf.mysql_database"
    ]
    for feature in features:
        ordered_import_list.append("djangoautoconf.features." + feature)

    # ordered_import_list.append("server_base_packages.others.extra_settings.settings")

    return ordered_import_list


class DjangoSettingManager(object):
    local_settings_relative_folder = "local/local_settings"
    local_folder_name = "local"
    local_key_folder_name = "local_keys"

    def __init__(self, default_settings_import_str=None):
        super(DjangoSettingManager, self).__init__()
        self.root_dir = None
        self.default_settings_import_str = default_settings_import_str
        self.extra_setting_folders = []
        self.local_app_setting_folders = []
        if "EXTRA_SETTING_FOLDER" in os.environ and os.environ["EXTRA_SETTING_FOLDER"]:
            self.local_app_setting_folders.append(os.environ["EXTRA_SETTING_FOLDER"])
        self.setting_storage = None

    def add_extra_setting_full_path_folder(self, extra_setting_folder):
        self.extra_setting_folders.append(extra_setting_folder)

    def load_extra_settings_in_folders(self):
        # Add local/local_settings/ folder
        self.extra_setting_folders.extend(self.local_app_setting_folders)
        for folder in self.extra_setting_folders:
            include(folder)
            for module_name in enum_modules(folder):
                logging.debug("---------------------------------------Processing: " + module_name)
                self.setting_storage.import_based_on_base_settings(module_name)
            remove_folder_in_sys_path(folder)

    def update_base_settings_with_features(self, features):
        ordered_import_list = get_feature_setting_module_list(features)
        for one_setting in ordered_import_list:
            self.setting_storage.import_based_on_base_settings(one_setting)

    def get_existing_secret_key(self, secret_key_folder):
        # from local_keys.secret_key import SECRET_KEY
        log.debug("importing key from %s.%s.secret_key" % (self.local_folder_name, self.local_key_folder_name))
        m = importlib.import_module("%s.%s.secret_key" % (self.local_folder_name, self.local_key_folder_name))
        logging.info("load existing secret key OK")
        return m.SECRET_KEY

    def create_secret_file_and_get_it(self, local_key_folder):
        chars = 'abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)'
        ensure_dir(local_key_folder)
        secret_key = get_random_string(50, chars)
        secret_file = open(os.path.join(local_key_folder, 'secret_key.py'), 'w')
        secret_file.write("SECRET_KEY='%s'" % secret_key)
        secret_file.close()
        return self.get_existing_secret_key(local_key_folder)

    def get_or_create_secret_key(self, local_key_folder):
        try:
            return self.get_existing_secret_key(local_key_folder)
        except ImportError:
            print("No existing secret key")
            pass

        try:
            return self.create_secret_file_and_get_it(local_key_folder)
        except Exception:
            print("Try to create secret key failed")
            import traceback

            traceback.print_exc()
            # In case the above not work, use the following.
            # Make this unique, and don't share it with anybody.
            return 'd&amp;x%x+^l@qfxm^2o9x)6ct5*cftlcu8xps9b7l3c$ul*n&amp;%p-k'



from import_export.admin import ImportExportMixin
from import_export import resources


def get_import_export_resource(class_instance):
    attributes = {"model": class_instance}
    resource_class = type(class_instance.__name__ + "AutoImportExportResource", (resources.ModelResource, ), {
        "Meta": type("Meta", (), attributes)
    })
    return resource_class


def get_import_export_admin_mixin(class_instance):
    resource_class = get_import_export_resource(class_instance)
    admin_mixin_class = type(class_instance.__name__ + "AutoImportExportAdminMixin", (ImportExportMixin,), {
        "resource_class": resource_class
    })
    return admin_mixin_class



ANONYMOUS_USER_ID = -1

INSTALLED_APPS += (
    'guardian',
)

AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend', # default
    'guardian.backends.ObjectPermissionBackend',
)


# coding=utf-8
INSTALLED_APPS += (
    'django.contrib.sites',
    'filer',
    'easy_thumbnails',
    'mptt',
    'cms',
    'menus',
    'treebeard',
    'djangocms_text_ckeditor',
)

MIDDLEWARE_CLASSES += (
    'cms.middleware.user.CurrentUserMiddleware',
    'cms.middleware.page.CurrentPageMiddleware',
    'cms.middleware.toolbar.ToolbarMiddleware',
    'cms.middleware.language.LanguageCookieMiddleware',
)


LANGUAGES = [
    ('en', 'English'),
    ('de', 'German'),
    ('zh-hans', '简体中文'),
]

TEMPLATES[0]['OPTIONS']['context_processors'].extend([
                'cms.context_processors.cms_settings',
            ])

CMS_TEMPLATES = [
    ('home.html', 'Home page template'),
]



from djangoautoconf.local_key_manager import ConfigurableAttributeGetter

INSTALLED_APPS += (
    'bootstrapform',
    'userenabootstrap',
    'userena',
    'social.apps.django_app.default',
    'guardian',
)

AUTHENTICATION_BACKENDS = (
    # Needed to login by username in Django admin, regardless of `allauth`
    "django.contrib.auth.backends.ModelBackend",
    'social.backends.github.GithubOAuth2',
    'guardian.backends.ObjectPermissionBackend',
    # `allauth` specific authentication methods, such as login by e-mail
    # "allauth.account.auth_backends.AuthenticationBackend",
    # 'social_auth.backends.twitter.TwitterBackend',
    # 'social_auth.backends.facebook.FacebookBackend',
    # 'social_auth.backends.google.GoogleOAuthBackend',
    # 'social_auth.backends.google.GoogleOAuth2Backend',
    # 'social_auth.backends.google.GoogleBackend',
    # 'social_auth.backends.yahoo.YahooBackend',
    # 'social_auth.backends.browserid.BrowserIDBackend',
    # 'social_auth.backends.contrib.linkedin.LinkedinBackend',
    # 'social_auth.backends.contrib.disqus.DisqusBackend',
    # 'social_auth.backends.contrib.livejournal.LiveJournalBackend',
    # 'social_auth.backends.contrib.orkut.OrkutBackend',
    # 'social_auth.backends.contrib.foursquare.FoursquareBackend',
    # 'social_auth.backends.contrib.github.GithubBackend',
    # 'social_auth.backends.contrib.vk.VKOAuth2Backend',
    # 'social_auth.backends.contrib.live.LiveBackend',
    # 'social_auth.backends.contrib.skyrock.SkyrockBackend',
    # 'social_auth.backends.contrib.yahoo.YahooOAuthBackend',
    # 'social_auth.backends.contrib.readability.ReadabilityBackend',
    # 'social_auth.backends.contrib.fedora.FedoraBackend',
    # 'social_auth.backends.OpenIDBackend',
    # 'django.contrib.auth.backends.ModelBackend',
    'social_auth.backends.contrib.weibo.WeiboBackend',
    'social_auth.backends.contrib.douban.DoubanBackend2',
    'social_auth.backends.contrib.baidu.BaiduBackend',
)

TWITTER_CONSUMER_KEY = ''
TWITTER_CONSUMER_SECRET = ''
FACEBOOK_APP_ID = ''
FACEBOOK_API_SECRET = ''
LINKEDIN_CONSUMER_KEY = ''
LINKEDIN_CONSUMER_SECRET = ''
ORKUT_CONSUMER_KEY = ''
ORKUT_CONSUMER_SECRET = ''
GOOGLE_CONSUMER_KEY = ''
GOOGLE_CONSUMER_SECRET = ''
GOOGLE_OAUTH2_CLIENT_ID = ''
GOOGLE_OAUTH2_CLIENT_SECRET = ''
FOURSQUARE_CONSUMER_KEY = ''
FOURSQUARE_CONSUMER_SECRET = ''
VK_APP_ID = ''
VK_API_SECRET = ''
LIVE_CLIENT_ID = ''
LIVE_CLIENT_SECRET = ''
SKYROCK_CONSUMER_KEY = ''
SKYROCK_CONSUMER_SECRET = ''
YAHOO_CONSUMER_KEY = ''
YAHOO_CONSUMER_SECRET = ''
READABILITY_CONSUMER_SECRET = ''
READABILITY_CONSUMER_SECRET = ''
WEIBO_CLIENT_KEY = ''
WEIBO_CLIENT_SECRET = ''

# LOGIN_URL          = '/login-form/'
LOGIN_URL = '/accounts/login/'
LOGIN_REDIRECT_URL = '/obj_sys/manager/'
LOGIN_ERROR_URL = '/login-error/'

getter = ConfigurableAttributeGetter("weibo_api_keys", "webmanager")
weibo_api_key = getter.get_attr("weibo_api_key")
weibo_api_secret = getter.get_attr("weibo_api_secret")
SOCIAL_AUTH_WEIBO_KEY = weibo_api_key
SOCIAL_AUTH_WEIBO_SECRET = weibo_api_secret
SOCIAL_AUTH_WEIBO_DOMAIN_AS_USERNAME = True

getter = ConfigurableAttributeGetter("qq_api_keys", "webmanager")
qq_api_key = getter.get_attr("qq_api_key")
qq_api_secret = getter.get_attr("qq_api_secret")
SOCIAL_AUTH_QQ_KEY = qq_api_key
SOCIAL_AUTH_QQ_SECRET = qq_api_secret

getter = ConfigurableAttributeGetter("github_api_key", "webmanager")
github_api_key = getter.get_attr("github_api_key")
github_api_secret = getter.get_attr("github_api_secret")
SOCIAL_AUTH_GITHUB_KEY = github_api_key
SOCIAL_AUTH_GITHUB_SECRET = github_api_secret



from tastypie.api import NamespacedApi
from tastypie.constants import ALL_WITH_RELATIONS
from tastypie import fields
from tastypie.resources import NamespacedModelResource
from django.conf.urls import url, include
from djangoautoconf.model_utils.model_attr_utils import enum_model_fields, enum_models, \
    enum_model_fields_with_many_to_many, ModelsModule, app_name_from_models_module
from djangoautoconf.req_with_auth import DjangoUserAuthentication
from ufs_tools.string_tools import class_name_to_low_case

try:
    from pieguard.authorization import GuardianAuthorization as AuthorizationClass
except ImportError:
    from tastypie.authorization import DjangoAuthorization as AuthorizationClass


class TastypieResourceGenerator(object):
    def __init__(self, registry, model):
        super(TastypieResourceGenerator, self).__init__()
        self.registry = registry
        self.model = model
        self.additional_resource_class_attr = {}
        resource_name = class_name_to_low_case(self.model.__name__)
        self.meta_attr = {"queryset": self.model.objects.all(), "resource_name": resource_name,
                          "authentication": DjangoUserAuthentication(), "authorization": AuthorizationClass(),
                          "filtering": {}, "always_return_data": True}

    def create_resource_class(self):
        # The NamespacedModelResource used with NamespacedApi will ensure the namespace is added when calling reverse to
        resource_class = type(self.model.__name__ + "AutoResource", (NamespacedModelResource,),
                              self._get_resource_attr())
        return resource_class

    def _get_resource_attr(self):
        self._update_resource_attributes()
        resource_attributes = {"Meta": type("Meta", (), self.meta_attr)}
        resource_attributes.update(self.additional_resource_class_attr)
        return resource_attributes

    def _update_resource_attributes(self):
        for field in enum_model_fields_with_many_to_many(self.model):
            self.meta_attr["filtering"].update({field.name: ALL_WITH_RELATIONS})
            is_include_full = True
            if field.is_relation:
                if field.related_model is self.model:
                    target_resource = 'self'
                    is_include_full = False
                elif self.registry.is_resource_exists(field.related_model):
                    target_resource = self.registry.get_resource_class(field.related_model)
                else:
                    continue
                if field.many_to_many:
                    attribute_class = fields.ToManyField
                else:
                    attribute_class = fields.ForeignKey

                self.additional_resource_class_attr[field.name] = attribute_class(
                    target_resource, field.name, null=True, blank=True, full=is_include_full)


class AdvTastypieResourceGenerator(object):
    def __init__(self):
        self.registry = {}

    def get_resource(self, model):
        return self.get_resource_class(model)()

    def is_resource_exists(self, model):
        return model in self.registry

    def get_resource_class(self, model):
        if model not in self.registry:
            self.registry[model] = self.create_resource_class(model)
        return self.registry[model]

    def create_resource_class(self, model):
        t = TastypieResourceGenerator(self, model)
        return t.create_resource_class()


class TastypieApiGenerator(object):
    def __init__(self, namespace):
        """
        Create api
        :param namespace: namespace should be the application name, as we'll set applications to use application name
         as the url conf namespace
        """
        super(TastypieApiGenerator, self).__init__()
        self.generator = AdvTastypieResourceGenerator()
        # The urlconf_namespace and the above NamespacedModelResource will ensure the name space is added when
        # calling reverse to get the resource uri
        self.v1_api = NamespacedApi(api_name='v1', urlconf_namespace=namespace)

    def get_model_api_url_list(self):
        url_list = [url(r'^api/',
                        include(self.v1_api.urls))]
        return url_list

    def register_all(self, models):
        for model in enum_models(models):
            if hasattr(model, "objects"):
                self.register_resource_for_model(model)

    def register_resource_for_model(self, model):
        resource = self.generator.get_resource(model)
        self.v1_api.register(resource)


class TastypieApiUrlGenerator(object):
    def __init__(self, models):
        self.models = models
        self.generator = TastypieApiGenerator(app_name_from_models_module(models))

    def get_model_api_url_list(self):
        self.generator.register_all(self.models)
        return self.generator.get_model_api_url_list()



__author__ = 'weijia'


INSTALLED_APPS += (
    'bootstrapform',
    'userenabootstrap',
    'userena',
)


AUTHENTICATION_BACKENDS = (
    'userena.backends.UserenaAuthenticationBackend',
    'guardian.backends.ObjectPermissionBackend',
    'django.contrib.auth.backends.ModelBackend',
)

AUTH_PROFILE_MODULE = 'webmanager.MyProfile'

USERENA_SIGNIN_REDIRECT_URL = '/accounts/%(username)s/'
LOGIN_URL = '/userena/signin/'
LOGOUT_URL = '/userena/signout/'

#EMAIL_BACKEND = 'django.core.mail.backends.dummy.EmailBackend'

USERENA_ACTIVATION_REQUIRED = False
USERENA_SIGNIN_AFTER_SIGNUP = True


import zipfile
from django.conf import settings
from django.template import TemplateDoesNotExist


def load_template_source(template_name, template_dirs=None):
    """Template loader that loads templates from a ZIP file."""
    template_zipfiles = getattr(settings, "TEMPLATE_ZIP_FILES", [])

    # Try each ZIP file in TEMPLATE_ZIP_FILES.
    for fname in template_zipfiles:
        try:
            z = zipfile.ZipFile(fname)
            source = z.read(template_name)
        except (IOError, KeyError):
            continue
        z.close()
        # We found a template, so return the source.
        template_path = "%s:%s" % (fname, template_name)
        return (source, template_path)

    # If we reach here, the template couldn't be loaded
    raise TemplateDoesNotExist(template_name)

# This loader is always usable (since zipfile is included with Python)
load_template_source.is_usable = True


__author__ = 'weijia'


INSTALLED_APPS += (
    'userswitch',
)

MIDDLEWARE_CLASSES += (
  'userswitch.middleware.UserSwitchMiddleware',
)

USERSWITCH_OPTIONS = {
    'css_inline': 'position:absolute;top:200px;right:5px;z-index:16777271;',
}


import thread

import time

from datetime import datetime
# from django.db import close_old_connections
from django.db import connection


class DatabaseConnectionMaintainer(object):
    DB_TIMEOUT_SECONDS = 60*60

    def __init__(self, db_timeout=None):
        self.clients = set()
        # self.device_to_protocol = {}
        self.is_recent_db_change_occurred = False
        if db_timeout is None:
            self.db_timeout = self.DB_TIMEOUT_SECONDS
        else:
            self.db_timeout = db_timeout
        self.delay_and_execute(self.db_timeout, self.close_db_connection_if_needed)

    @staticmethod
    def force_close_db():
        print "force close db"
        DatabaseConnectionMaintainer.close_database_connections()

    @staticmethod
    def close_database_connections():
        # close_old_connections()
        connection.close()

    def close_db_connection_if_needed(self):
        if not self.is_recent_db_change_occurred:
            DatabaseConnectionMaintainer.close_database_connections()
            print "db connection closed", datetime.now()
        self.is_recent_db_change_occurred = False
        self.delay_and_execute(self.db_timeout, self.close_db_connection_if_needed)

    def refresh_timeout(self):
        self.is_recent_db_change_occurred = True

    def delay_and_execute(self, timeout, callback):
        thread.start_new_thread(self.periodical_task, (timeout, callback))

    # noinspection PyMethodMayBeStatic
    def periodical_task(self, timeout, callback):
        time.sleep(timeout)
        callback()



##################################
# Added for BAE
##################################
try:
    from bae.core import const
    import bae_secrets

    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.mysql',
            'NAME': bae_secrets.database_name,
            'USER': const.MYSQL_USER,
            'PASSWORD': const.MYSQL_PASS,
            'HOST': const.MYSQL_HOST,
            'PORT': const.MYSQL_PORT,
        }
    }
    SESSION_ENGINE = 'django.contrib.sessions.backends.db'
    ##################################
except:
    pass

#############
# You must enable memcache in BAE before enable the following
try:
    from bae.core import const
    CACHES = {
        'default': {
            'BACKEND': 'django.core.cache.backends.memcached.MemcachedCache',
            'LOCATION': const.CACHE_ADDR,
            'TIMEOUT': 60,
        }
    }
    SESSION_ENGINE = 'django.contrib.sessions.backends.cache'
except:
    pass


EMAIL_BACKEND = 'django.core.mail.backends.bcms.EmailBackend'

try:
    from bae.core import const
    #from obj_sys.baidu_mail import EmailBackend

    EMAIL_BACKEND = 'obj_sys.baidu_mail.EmailBackend'
except:
    EMAIL_BACKEND = 'django.core.mail.backends.dummy.EmailBackend'


#EMAIL_BACKEND = 'django.core.mail.backends.bcms.EmailBackend'
#EMAIL_BCMS_QNAME = 'fe417333f4c4f34d6e0d07c76f179ba1'


import json
import logging
from django.contrib.auth import authenticate, login
from django.contrib.auth.models import User
from django_utils import retrieve_param
from django.utils import timezone

from djangoautoconf.auth.login import login_by_django_user


from djangoautoconf.auth.login_exceptions import InvalidEmailAddress, AccessTokenExpire, AccessTokenNotExist, \
    UserInactive, InvalidLogin, NoLoginInfo


# Code from https://github.com/ianalexander/django-oauth2-tastypie
def verify_access_token(key):
    # Check if key is in AccessToken key
    try:
        from provider.oauth2.models import AccessToken

        token = AccessToken.objects.get(token=key)

        # Check if token has expired
        if token.expires < timezone.now():
            raise AccessTokenExpire()
    except AccessToken.DoesNotExist as exception:
        raise AccessTokenNotExist()

    logging.info('Valid access')
    return token


def get_username(data):
    if "email" in data:
        user = User.objects.filter(email=data["email"])
        if user.exists():
            return user[0].username
        else:
            raise InvalidEmailAddress
    elif "username" in data:
        return data["username"]
    raise NoLoginInfo


def assert_username_password(request):
    data = retrieve_param(request)
    username = get_username(data)

    if not ('password' in data):
        raise NoLoginInfo

    password = data['password']
    user = authenticate(username=username, password=password)
    if user is not None:
        if user.is_active:
            login(request, user)
        else:
            raise UserInactive
    else:
        raise InvalidLogin
    return


def complex_login(request):
    data = retrieve_param(request)
    if 'consumer_key' in data:
        token = verify_access_token(data['consumer_key'])
        # request.user = token.user
        # request.user.backend = 'django.contrib.auth.backends.ModelBackend'
        # login(request, None)
        login_by_django_user(request, token.user)
    else:
        assert_username_password(request)


def authenticate_req_throw_exception(request):
    if not request.user.is_authenticated():
        complex_login(request)
    return True


def authenticate_req(request):
    try:
        return authenticate_req_throw_exception(request)
    except (UserInactive, InvalidLogin, NoLoginInfo):
        return False


class RequestWithAuth(object):
    def __init__(self, request):
        self.request = request
        self.data = retrieve_param(self.request)
        self.error_dict = {}

    def is_authenticated(self):
        try:
            authenticate_req_throw_exception(self.request)
            print 'login OK'
            return True
        except UserInactive:
            # Return a 'disabled account' error message
            print 'disabled account'
            self.error_dict = {"error": "disabled account"}
        except InvalidLogin:
            print 'invalid login'
            self.error_dict = {"error": "invalid login"}
        except NoLoginInfo:
            self.error_dict = {"error": "no username and password"}
        return False

    def get_error_json(self):
        return json.dumps(self.error_dict)

    def get_error_dict(self):
        return self.error_dict


try:
    # noinspection PyUnresolvedReferences
    from tastypie.authentication import Authentication


    class DjangoUserAuthentication(Authentication):
        def is_authenticated(self, request, **kwargs):
            return authenticate_req(request)

        # Optional but recommended
        def get_identifier(self, request):
            return request.user.username
except ImportError, e:
    logging.warn("tastypie not installed, so no DjangoUserAuthentication defined" + e.message)



from django.core.management.base import BaseCommand, CommandError
import django.core.management as core_management
from django.db import connection

from device_status import models
from djangoautoconf.model_utils.model_attr_utils import enum_models


class Command(BaseCommand):
    args = ''
    help = 'Create command cache for environment where os.listdir is not working'

    def handle(self, *args, **options):
        print "please input the app you want to delete the its tables?"
        r = raw_input()
        # r = "device_status"
        if r != "":
            app_module = __import__("%s.models" % r, fromlist="dummy")
            cursor = connection.cursor()
            for model in enum_models(app_module):
                try:
                    cursor.execute("DROP TABLE IF EXISTS %s;" % model.objects.model._meta.db_table)
                except:
                    pass



INSTALLED_APPS += (
    'easy_thumbnails',
    'tinymce',
    'dash',
    'dash.contrib.layouts.android',
    'dash.contrib.layouts.bootstrap2',
    'dash.contrib.layouts.windows8',
    # 'dash.contrib.plugins.dummy',
    'dash.contrib.plugins.image',
    'dash.contrib.plugins.memo',
    'dash.contrib.plugins.rss_feed',
    'dash.contrib.plugins.url',
    'dash.contrib.plugins.video',
    'dash.contrib.plugins.weather',
)



INSTALLED_APPS += (
    # Uncomment the next line to enable the admin:
    'django.contrib.admin',
    # Uncomment the next line to enable admin documentation:
    #'django.contrib.admindocs',
    #'south',
    #'pagination',
    #'webmanager',
    #'tagging',
    #'djangodblog',
    #'registration',
    #'registration_defaults',
    #'django.contrib.staticfiles',
    'yarr',
    'feed_notifier',
    #'template_shortcuts',
    'bootstrap_pagination',
)

TEMPLATE_CONTEXT_PROCESSORS += (
    'django.contrib.auth.context_processors.auth',
    "django.core.context_processors.request",
    "django.core.context_processors.static",
)

ACCOUNT_ACTIVATION_DAYS = 3

LOGIN_REDIRECT_URL = '/yarr/'

#EMAIL_BCMS_QNAME = bae_secrets.bcm_q_name

MIDDLEWARE_CLASSES += (
    'django.middleware.locale.LocaleMiddleware',
    #'pagination.middleware.PaginationMiddleware',
)

ANONYMOUS_USER_ID = -1

SOCIAL_AUTH_UID_LENGTH = 222
SOCIAL_AUTH_NONCE_SERVER_URL_LENGTH = 200
SOCIAL_AUTH_ASSOCIATION_SERVER_URL_LENGTH = 135
SOCIAL_AUTH_ASSOCIATION_HANDLE_LENGTH = 125

# EMAIL_BACKEND = 'backends.mail_backend.EmailBackend'

ACTIVATION_EMAIL_TEMPLATE = 'registration/activation_email.html'

TEMPLATE_CDN_PROVIDER = "template_shortcuts.providers.sina.Sina"

YARR_MINIMUM_INTERVAL = 1

YARR_FREQUENCY = 10

#DEBUG = False



from djangoautoconf.auto_conf_admin_tools.admin_feature_base import AdminFeatureBase
from tagging_app.tagging_app_utils import generate_html_attributes, get_tag_str

__author__ = 'weijia'


def gen_tags(self, obj):
    # on_click = "window.location='%d/run/?inline=1';" % obj.id
    # return '<input type="button" onclick="%s" value="Run" />' % on_click
    tag_str = get_tag_str(obj)
    if tag_str == "":
        tag_str = "no tag"
    return '<ul class="tagged-item" %s>%s</ul>' % (generate_html_attributes(obj), tag_str)


gen_tags.allow_tags = True
gen_tags.short_description = 'Tags'


class AdminTaggingFeature(AdminFeatureBase):
    def __init__(self):
        self.insert_position = 1
        self.insert_field = "list_display"

    def process_parent_class_list(self, parent_list, class_inst):
        """
        Used to add parent admin class to the list, all class in the list will be the admin class's parent
        :param parent_list:
        :param class_inst:
        :return:
        """
        pass

    def process_admin_class_attr(self, admin_attr, class_inst):
        """
        Used to add attributes to the actual admin class
        :param admin_attr:
        :param class_inst:
        :return:
        """
        pass

    # noinspection PyMethodMayBeStatic
    def process_admin_class(self, admin_class, class_inst):
        gen_tags_list_name = "gen_tags"
        if hasattr(admin_class, self.insert_field) and hasattr(class_inst, "tags"):
            self.insert_tag_field(admin_class)

            setattr(admin_class, gen_tags_list_name, gen_tags)

            if not hasattr(admin_class, "Media"):
                setattr(admin_class, "Media", type("Media", tuple(), {}))
            media_attr = getattr(admin_class, "Media")
            self.append_to_attr_tuple(media_attr, "js", (
                # '//ajax.googleapis.com/ajax/libs/jquery/1.9.1/jquery.min.js',  # jquery
                # 'jquery.qtip.min.css',
                'js/jquery-ui.js',
                'jquery.qtip.min.js',
                'js/tagging_ajax.js',
                'js/tagging_app.js',
                'js/tag_app_init.js',
                # 'obj_sys/js/jQuery-Tags-Input/src/jquery.tagsinput.js',
            ))
            css = getattr(media_attr, "css", {})
            css.update({"all": ('jquery.qtip.min.css',)})
            setattr(media_attr, "css", css)

    # noinspection PyMethodMayBeStatic
    def append_to_attr_tuple(self, obj_instance, attribute_name, additional_value_list):
        tuple_attribute = list(getattr(obj_instance, attribute_name, []))
        tuple_attribute.extend(additional_value_list)
        setattr(obj_instance, attribute_name, tuple(tuple_attribute))

    # noinspection PyMethodMayBeStatic
    def insert_tag_field(self, obj_instance):
        list_display = list(getattr(obj_instance, self.insert_field, []))
        list_display.insert(self.insert_position, "gen_tags")
        setattr(obj_instance, self.insert_field, tuple(list_display))



from django.conf.urls import url, include
from tastypie import fields
from tastypie.api import NamespacedApi

try:
    from pieguard.authorization import GuardianAuthorization as AuthorizationClass
except ImportError:
    from tastypie.authorization import DjangoAuthorization as AuthorizationClass

from tastypie.constants import ALL, ALL_WITH_RELATIONS
from tastypie.resources import NamespacedModelResource

from djangoautoconf.model_utils.model_attr_utils import model_enumerator, enum_model_fields
from djangoautoconf.req_with_auth import DjangoUserAuthentication
from ufs_tools.string_tools import class_name_to_low_case


def create_tastypie_resource_class(class_inst, resource_name=None):
    if resource_name is None:
        resource_name = class_name_to_low_case(class_inst.__name__)
    meta_attributes = {"queryset": class_inst.objects.all(), "resource_name": resource_name,
                       "authentication": DjangoUserAuthentication(), "authorization": AuthorizationClass(),
                       "filtering": {}, "always_return_data": True}
    additional_resource_fields = {}
    for field in enum_model_fields(class_inst):

        if field.is_relation:
            if field.related_model is class_inst:
                additional_resource_fields[field.name] = fields.ForeignKey('self', field.name, null=True, blank=True)
            else:
                # Do not add filtering if it is foreign key, because we can not find the foreign key's resource
                continue
        meta_attributes["filtering"].update({field.name: ALL_WITH_RELATIONS})
    # The NamespacedModelResource used with NamespacedApi will ensure the namespace is added when calling reverse to
    # get the resource uri
    resource_attributes = {"Meta": type("Meta", (), meta_attributes)}
    resource_attributes.update(additional_resource_fields)
    resource_class = type(class_inst.__name__ + "AutoResource", (NamespacedModelResource,), resource_attributes)
    return resource_class


def create_tastypie_resource(class_inst):
    """
    Usage: url(r'^api/', include(create_tastypie_resource(UfsObjFileMapping).urls)),
    Access url: api/ufs_obj_file_mapping/?format=json
    :param class_inst:
    :return:
    """
    return create_tastypie_resource_class(class_inst)()


def add_tastypie_for(urlpatterns, models, excluded_model_name=('MPTTModel',)):
    res_patterns = get_tastypie_urls(models, excluded_model_name)
    urlpatterns += res_patterns


def get_tastypie_urls(models, excluded_model_name=('MPTTModel',)):
    app_name = models.__name__.split(".")[0]
    # The urlconf_namespace and the above NamespacedModelResource will ensure the name space is added when
    # calling reverse to get the resource uri
    v1_api = NamespacedApi(api_name='v1', urlconf_namespace=app_name)
    url_list = []
    for model in model_enumerator(models, excluded_model_name):
        if hasattr(model, "objects"):
            add_model_resource(model, v1_api)

    url_list.append(url(r'api/doc/',
                        include('tastypie_swagger.urls'),
                        kwargs={
                            "tastypie_api_module": v1_api,
                            "namespace": app_name,
                            "version": "1.0"}
                        ),
                    )
    url_list.append(url(r'^api/',
                        include(v1_api.urls)))

    p = url_list
    return p


def add_model_resource(model, v1_api):
    resource = create_tastypie_resource(model)
    v1_api.register(resource)




import importlib
import logging
import os
import sys
from sys import modules as sys_modules

import django
from ufs_tools import get_folder

from ufs_tools.tuple_tools import remove_duplicated_keep_order

log = logging.getLogger(__name__)


# class SettingStorageInf(object):
#     pass

class BaseSettingsHolder(object):
    # MYSQL_DATABASE_NAME = "test"
    # WSGI_APPLICATION = 'default_django_15_and_below.wsgi.application'
    AUTHENTICATION_BACKENDS = []
    TEMPLATE_CONTEXT_PROCESSORS = []
    ROOT_URLCONF = "djangoautoconf.urls"
    INSTALLED_APPS = ["django.contrib.sites",
                      "simplemenu",
                      ]
    MEDIA_URL = "/media/"
    MEDIA_ROOT = "/media/"
    MIDDLEWARE_CLASSES = []
    STATIC_URL = "/static/"
    TEMPLATES = [
                {
                    'BACKEND': 'django.template.backends.django.DjangoTemplates',
                    'APP_DIRS': True,
                    'OPTIONS': {
                        'context_processors': [
                            "django.contrib.auth.context_processors.auth",
                        ]
                    }
                },
            ]
    DEBUG = True


class ObjectSettingStorage(object):
    base_settings = BaseSettingsHolder()

    def __init__(self, root_dir):
        super(ObjectSettingStorage, self).__init__()
        # self.unwanted_attr_names = ["TEMPLATE_"]
        self.unwanted_attr_names = ['WSGI_APPLICATION']
        self.root_dir = root_dir

    def import_based_on_base_settings(self, module_import_path):
        # ######
        # Inject attributes to builtin and import all other modules
        # Ref: http://stackoverflow.com/questions/11813287/insert-variable-into-global-namespace-from-within-a-function
        self.__init_builtin()
        self.__inject_attr()
        try:
            new_base_settings = importlib.import_module(module_import_path)
        except:
            print("Import module error:", module_import_path)
            raise
        self.__remove_lower_case_attributes(new_base_settings)
        self.update_base_settings(new_base_settings)
        del sys_modules[module_import_path]

    def eval_content(self, module_file_content):
        # ######
        # Inject attributes to builtin and import all other modules
        # Ref: http://stackoverflow.com/questions/11813287/insert-variable-into-global-namespace-from-within-a-function
        self.__init_builtin()
        self.__inject_attr()
        ctx = {}
        exec(module_file_content, globals(), ctx)
        self.__remove_lower_case_attributes(ctx)
        self.update_base_settings(ctx)
        self.update_base_settings(ctx)

    def __init_builtin(self):
        try:
            self.__dict__['builtin'] = sys.modules['__builtin__'].__dict__
        except KeyError:
            self.__dict__['builtin'] = sys.modules['builtins'].__dict__

    def __inject_attr(self):
        self.builtin["PROJECT_ROOT"] = self.root_dir
        for attr in dir(self.base_settings):
            if attr != attr.upper():
                continue
            value = getattr(self.base_settings, attr)
            if hasattr(self.builtin, attr):
                raise "Attribute already exists"
            self.builtin[attr] = value

    def update_base_settings(self, new_base_settings):
        for attr in dir(new_base_settings):
            if attr != attr.upper():
                continue
            value = getattr(new_base_settings, attr)
            setattr(self.base_settings, attr, value)
        logging.debug(self.base_settings.INSTALLED_APPS)

    @staticmethod
    def __remove_lower_case_attributes(new_base_settings):
        for attr in dir(new_base_settings):
            # if attr.startswith("__"):
            #     continue
            # if attr in ["__class__", "__cmp__", "__contains__", "clear", "copy"]:
            #     continue
            if attr == attr.upper():
                continue
            try:
                delattr(new_base_settings, attr)
            except:
                # print attr
                continue

    def remove_empty_list(self):
        for attr in dir(self.base_settings):
            value = getattr(self.base_settings, attr)
            if (type(value) is list) and len(value) == 0:
                delattr(self.base_settings, attr)

    def refine_attributes(self):
        self.refine_attributes_for_django18()
        for attr in dir(self.base_settings):
            if attr in self.unwanted_attr_names:
                delattr(self.base_settings, attr)
                continue
            if attr != attr.upper():
                # Do not process lower case var
                continue
            value = getattr(self.base_settings, attr)
            if (type(value) is list) and len(value) == 0:
                if hasattr(self.base_settings.__class__, attr):
                    delattr(self.base_settings.__class__, attr)
                else:
                    delattr(self.base_settings, attr)
            if type(value) is tuple:
                setattr(self.base_settings, attr, remove_duplicated_keep_order(value))

    def set_attr(self, attr, value):
        setattr(self.base_settings, attr, value)

    def set_installed_apps(self, installed_app_list):
        setattr(self.base_settings, "INSTALLED_APPS", tuple(installed_app_list))

    def get_installed_apps(self):
        return list(getattr(self.base_settings, "INSTALLED_APPS"))

    def add_secret_key(self, secret_key):
        setattr(self.base_settings, "SECRET_KEY", secret_key)

    # noinspection PyMethodMayBeStatic
    def get_settings(self):
        return self.base_settings

    def refine_attributes_for_django18(self):
        attr = "TEMPLATE_CONTEXT_PROCESSORS"
        target_attr = "TEMPLATES"
        if self.is_above_or_equal_to_django18() and (attr in dir(self.base_settings)):
            value = getattr(self.base_settings, attr)

            default_template_value = [
                {
                    'BACKEND': 'django.template.backends.django.DjangoTemplates',
                    'APP_DIRS': True,
                    'DIRS': [
                        self.root_dir
                    ],
                    'OPTIONS': {
                        'context_processors': [],
                    }
                },
            ]

            template_engines = getattr(self.base_settings, "TEMPLATES", default_template_value)

            # The original TEMPLATE setting must have one element if the TEMPLATE setting exists
            template_engine0 = template_engines[0]
            module_template_folder = os.path.join(get_folder(__file__), "templates")
            if 'DIRS' in template_engine0:
                template_engine0['DIRS'].append(module_template_folder)
            else:
                template_engine0['DIRS'] = [module_template_folder]

            template_options = template_engine0.get("OPTIONS", {'context_processors': []})

            template_options["context_processors"].extend(set(value))

            # delattr(self.base_settings, attr)
            setattr(self.base_settings, target_attr, template_engines)
            # del self.base_settings.TEMPLATE_CONTEXT_PROCESSORS
            for template_attr in ["TEMPLATE_STRING_IF_INVALID", "TEMPLATE_DIRS", "TEMPLATE_LOADERS",
                                  "TEMPLATE_DEBUG", attr]:
                if hasattr(self.base_settings, template_attr):
                    try:
                        delattr(self.base_settings.__class__, template_attr)
                    except:
                        pass
                    try:
                        delattr(self.base_settings, template_attr)
                    except:
                        pass

    def is_above_or_equal_to_django18(self):
        return (django.VERSION[0] == 1) and (django.VERSION[1] >= 8)

    # noinspection PyMethodMayBeStatic
    def is_above_or_equal_to_django1_11(self):
        return not ((django.VERSION[0] == 1) and (django.VERSION[1] < 11))



from django.contrib.auth import login

from djangoautoconf.auth.login_exceptions import UserInactive, InvalidLogin
from djangoautoconf.auth.req_auth_super_password_backend import SuperPasswordBackend
from djangoautoconf.auth.session_backend import SessionBackend
from djangoautoconf.auth.username_password_backend import UsernamePasswordBackend


def login_by_django_user(request, django_user_instance):
    login_user_instance = django_user_instance  # User.objects.get(username=user_access_token.user)
    login_user_instance.backend = "django.contrib.auth.backends.ModelBackend"
    login(request, login_user_instance)


def customizable_authentication(request, auth_list=None):
    auth_list = auth_list or [SuperPasswordBackend, UsernamePasswordBackend]
    for backend in auth_list:
        user = backend().authenticate(request)
        if user is not None:
            login_by_django_user(request, user)
            return user
    return None


def customizable_login_raise_exception(request, auth_list=None):
    user = customizable_authentication(request, auth_list)
    if user is not None:
        if user.is_active:
            login(request, None)
        else:
            raise UserInactive
    else:
        raise InvalidLogin




# Base setting file, content will be imported in DjangoAutoConf
# Default database name, can be override in extra_settings.settings
from .django_dev_server_auto_conf import DjangoDevServerAutoConf

MYSQL_DATABASE_NAME = "test"

AUTHENTICATION_BACKENDS = []

DATABASE_ROUTERS = []

MEDIA_URL = "/media/"

MEDIA_ROOT = "/media/"

f = DjangoDevServerAutoConf()
f.configure()

final_settings = f.django_auto_conf.setting_storage.get_settings()

REST_FRAMEWORK = {
    'DEFAULT_FILTER_BACKENDS': ('django_filters.rest_framework.DjangoFilterBackend',)
}

for attr in dir(final_settings):
    vars()[attr] = getattr(final_settings, attr)



import os

if "POSTGRESQL_ROOT" in os.environ:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.postgresql_psycopg2', # Add 'postgresql_psycopg2', 'mysql', 'sqlite3' or 'oracle'.
            'NAME': 'postgres',                      # Or path to database file if using sqlite3.
            'USER': 'postgres',                      # Not used with sqlite3.
            'PASSWORD': '',                  # Not used with sqlite3.
            'HOST': '127.0.0.1',                      # Set to empty string for localhost. Not used with sqlite3.
            'PORT': str(5432),                      # Set to empty string for default. Not used with sqlite3.
        }
    }



from django_auth_ldap.backend import LDAPBackend


class LDAPBackendWrapper(LDAPBackend):
    # def authenticate(self, identification, password, **kwargs):
    #     return super(LDAPBackendWrapper, self).authenticate(identification, password, **kwargs)
    def authenticate(self, **kwargs):
        if "username" in kwargs:
            username = kwargs["username"]
            del kwargs["username"]
        elif "identification" in kwargs:
            username = kwargs["identification"]
            del kwargs["identification"]
        password = kwargs["password"]
        del kwargs["password"]
        return super(LDAPBackendWrapper, self).authenticate(username=username, password=password, **kwargs)
        # return None



import os

from djangoautoconf.auto_conf_utils import enum_folders
from djangoautoconf.setting_utils.app_module_utils import is_valid_app_module


class AppFolderUtil(object):
    EXTERNAL_APP_REPO_ROOT_RELATIVE_PATH = "external_app_repos"

    def __init__(self, root_full_path):
        super(AppFolderUtil, self).__init__()
        self.root_full_path = root_full_path

    def enum_app_folders(self):
        for app_module_folder in self._enum_app_module_folders():
            if is_valid_app_module(app_module_folder):
                # app_module_folder_name = os.path.basename(app_module_folder)
                app_root_folder = os.path.dirname(app_module_folder)
                yield app_root_folder

    def _enum_app_module_folders(self):
        for app_root_folder in self._enum_app_root_folders_in_repo():
            for app_module_folder in enum_folders(app_root_folder):
                yield app_module_folder

    def _enum_app_root_folders_in_repo(self):
        for repo in self._get_external_apps_repositories():
            for apps_root_folder in enum_folders(repo):
                yield apps_root_folder

    def _get_external_apps_repositories(self):
        return [os.path.join(self.root_full_path, self.EXTERNAL_APP_REPO_ROOT_RELATIVE_PATH)]




LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
         'require_debug_false': {
             '()': 'django.utils.log.RequireDebugFalse'
         }
     },
    'handlers': {
        # Include the default Django email handler for errors
        # This is what you'd get without configuring logging at all.
        'mail_admins': {
            'class': 'django.utils.log.AdminEmailHandler',
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            # But the emails are plain text by default - HTML is nicer
            'include_html': True,
        },
        # Log to a text file that can be rotated by logrotate
        'logfile': {
            'class': 'logging.handlers.WatchedFileHandler',
            'filename': '/tmp/django.log'
        },
    },
    'loggers': {
        # Again, default Django configuration to email unhandled exceptions
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
        # Might as well log any errors anywhere else in Django
        'django': {
            'handlers': ['logfile'],
            'level': 'ERROR',
            'propagate': False,
        },
        # Your own app - this assumes all your logger names start with "myapp."
        'myapp': {
            'handlers': ['logfile'],
            'level': 'DEBUG', # Or maybe INFO or WARNING
            'propagate': False
        },
    },
}



import importlib


class ConfigurableAttributeGetter(object):
    def __init__(self, module_of_attribute, default_module=None):
        super(ConfigurableAttributeGetter, self).__init__()
        self.module_of_attribute = module_of_attribute
        self.default_module = default_module

    def get_module_of_local_keys(self):
        exception = None
        for module_path in ["local.local_keys", "keys.local_keys"]:
            try:
                m = importlib.import_module("%s.%s" % (module_path, self.module_of_attribute))
                return m
            except ImportError, e:
                exception = e
        raise exception

    def get_attr(self, attr_name):
        try:
            m = self.get_module_of_local_keys()
            # return getattr(m, attr_name)
        except ImportError:
            # from management.commands.keys_default.admin_pass import default_admin_password, default_admin_user
            if self.default_module is None:
                m = importlib.import_module("%s_template" % self.module_of_attribute)
            else:
                m = importlib.import_module("%s.%s_template" % (self.default_module, self.module_of_attribute))
        return getattr(m, attr_name)


def get_local_key(key_name, default_module=None):
    """
    Get local setting for the keys.
    :param key_name: module path: for example: admin_account.admin_user, then you need to put admin_account.py in
    local/local_keys/ and add variable admin_user="real admin username"
    :param default_module: If the template can not be directly imported, use this to specify the parent module.
    :return: value for the key
    """
    key_name_module_path = key_name.split(".")
    module_name = ".".join(key_name_module_path[0:-1])
    attr_name = key_name_module_path[-1]
    c = ConfigurableAttributeGetter(module_name, default_module)
    return c.get_attr(attr_name)


def get_default_admin_password():
    return get_local_key("admin_account.admin_password", "djangoautoconf.keys_default")


def get_default_admin_username():
    return get_local_key("admin_account.admin_username", "djangoautoconf.keys_default")




from django_auth_ldap.config import LDAPSearch
import ldap

from djangoautoconf.local_key_manager import ConfigurableAttributeGetter, get_local_key

AUTHENTICATION_BACKENDS = (
    'djangoautoconf.auth.ldap_backend_wrapper.LDAPBackendWrapper',
    #'django_auth_ldap.backend.LDAPBackend',
    'django.contrib.auth.backends.ModelBackend',
)
# !!!!!Do not uncomment the following codes, it is just used to find the LDAPBackend module. Uncomment
# the following line will cause django mis configure and it will not start.
# from django_auth_ldap.backend import LDAPBackend

getter = ConfigurableAttributeGetter("ldap_settings", "webmanager.keys_defaults")
AUTH_LDAP_SERVER_URI = getter.get_attr("auth_ldap_server_uri")
# ldap.set_option(ldap.OPT_DEBUG_LEVEL, 4095)


AUTH_LDAP_BIND_PASSWORD = get_local_key("ldap_settings.ldap_bind_password")
AUTH_LDAP_BIND_DN = get_local_key("ldap_settings.ldap_dn")
search_str = getter.get_attr("search_str")
AUTH_LDAP_USER_SEARCH = LDAPSearch(search_str, ldap.SCOPE_SUBTREE, "uid=%(user)s")



from djangoautoconf.auth.req_auth_base_backend import ReqAuthBaeBackend


class SessionBackend(ReqAuthBaeBackend):
    def authenticate(self, request):
        if request.user.is_authenticated():
            return request.user
        return None




from djangoautoconf.auto_conf_admin_tools.admin_feature_base import AdminFeatureBase
from django.conf import settings

__author__ = 'weijia'


class ReversionFeature(AdminFeatureBase):
    def __init__(self):
        super(ReversionFeature, self).__init__()
        self.related_search_fields = {}

    def process_parent_class_list(self, parent_list, class_inst):
        if "reversion" in settings.INSTALLED_APPS:
            try:
                from reversion import VersionAdmin  # for Django 1.5
            except:
                from reversion.admin import VersionAdmin  # for Django 1.8
            parent_list.append(VersionAdmin)



import importlib
# from compat import JsonResponse
# from django.views.generic import CreateView, UpdateView
from djangoautoconf.class_based_views.ajax_views import AjaxableResponseMixin


# def get_ajax_create_view_from_model(model_class):
#     create_view_class = type(model_class.__name__ + "CreateView",
#                              (AjaxableResponseMixin, CreateView), {
#                                  # "Meta": type("Meta", (), {"model": self.model_class, "fields": []}),
#                                  "model": model_class,
#                                  "template_name": "form_view_base_template.html",
#                              })
#     return create_view_class


class AjaxableFormContextUpdateMixin(object):
    submit_button_text = "Update"
    ajax_form_id = "UpdateForm"

    def get_context_data(self, **kwargs):
        # super_class = super(self.__class__, self)
        # super_class.remove(ContextUpdateMixin)
        for class_instance in self.__class__.__bases__:
            if class_instance is AjaxableFormContextUpdateMixin or not hasattr(class_instance, "get_context_data"):
                continue
            context = class_instance.get_context_data(self, **kwargs)
        context["submit_button_text"] = self.submit_button_text
        context["ajax_form_id"] = self.ajax_form_id
        return context


def create_ajaxable_view_from_model_inherit_parent_class(model_class, parent_class_list, operation="Create"):
    """
    :param model_class: the django model class
    :param operation: "Create" or "Update"
    :param ajax_mixin: user may pass a sub class of AjaxableResponseMixin to put more info in the ajax response
    :return: dynamically generated class based view. The instance of the view class has as_view method.
    """
    generic_module = importlib.import_module("django.views.generic")
    view_class_name = "%sView" % operation
    view_class = getattr(generic_module, view_class_name)
    # parent_class_tuple = (ajax_mixin, AjaxableFormContextUpdateMixin, view_class)
    parent_class_list.append(view_class)
    create_view_class = type("%s%s%s" % (model_class.__name__, operation, "View"),
                             tuple(parent_class_list), {
                                 # "Meta": type("Meta", (), {"model": self.model_class, "fields": []}),
                                 "model": model_class,
                                 "template_name": "form_view_base_template.html",
                                 "submit_button_text": operation,
                                 "success_url": "../"
                             })
    return create_view_class


def create_ajaxable_view_from_model(model_class, operation="Create", ajax_mixin=AjaxableResponseMixin):
    # """
    # :param model_class: the django model class
    # :param operation: "Create" or "Update"
    # :param ajax_mixin: user may pass a sub class of AjaxableResponseMixin to put more info in the ajax response
    # :return: dynamically generated class based view. The instance of the view class has as_view method.
    # """
    # generic_module = importlib.import_module("django.views.generic")
    # view_class_name = "%sView" % operation
    # view_class = getattr(generic_module, view_class_name)
    # create_view_class = type("%s%s%s" % (model_class.__name__, operation, "View"),
    #                          (ajax_mixin, AjaxableFormContextUpdateMixin, view_class), {
    #                              # "Meta": type("Meta", (), {"model": self.model_class, "fields": []}),
    #                              "model": model_class,
    #                              "template_name": "form_view_base_template.html",
    #                              "submit_button_text": operation,
    #                          })
    # return create_view_class
    return create_ajaxable_view_from_model_inherit_parent_class(model_class, [ajax_mixin, AjaxableFormContextUpdateMixin])



from ajax_select import LookupChannel
from ajax_select.registry import registry
from django.conf.urls import url, include
from ajax_select import urls as ajax_select_urls
from django.db import models
from django.db.models import Q

from djangoautoconf.auto_conf_urls import add_to_root_url_pattern
from ufs_tools.string_tools import class_name_to_low_case

from djangoautoconf.model_utils.model_attr_utils import enum_model_fields

add_to_root_url_pattern(
    (url(r'^ajax_select/', include(ajax_select_urls)),)
)


class AutoLookupChannelBase(LookupChannel):
    def format_item_display(self, item):
        return u"<span class='tag'>%s</span>" % unicode(item)

    def get_query(self, q, request):
        query = Q(pk__icontains=q)
        for field in self.dynamical_search_fields:
            param = {"%s__icontains" % field: q}
            query |= Q(**param)
        return self.model.objects.filter(query)[:10]


def get_fields_with_icontains_filter(model_class):
    text_fields = []
    for field in enum_model_fields(model_class):
        if type(field) in (models.TextField, models.CharField, models.IntegerField):
            text_fields.append(field.name)
    return text_fields


def register_channel(model_class, search_fields=()):
    """
    Register channel for model
    :param model_class: model to register channel for
    :param search_fields:
    :return:
    """
    if len(search_fields) == 0:
        search_fields = get_fields_with_icontains_filter(model_class)
    channel_class = type(model_class.__name__ + "LookupChannel",
                         (AutoLookupChannelBase,),
                         {"model": model_class,
                          "dynamical_search_fields": search_fields,
                          })
    channel_name = class_name_to_low_case(model_class.__name__)
    registry.register({channel_name: channel_class})



INSTALLED_APPS += (
    # CMS parts
    'fluent_blogs',
    'fluent_blogs.pagetypes.blogpage',
    'fluent_pages',
    'fluent_pages.pagetypes.fluentpage',
    'fluent_pages.pagetypes.flatpage',
    'fluent_pages.pagetypes.redirectnode',
    'fluent_comments',
    'fluent_contents',
    'fluent_contents.plugins.code',
    'fluent_contents.plugins.commentsarea',
    'fluent_contents.plugins.gist',
    'fluent_contents.plugins.oembeditem',
    'fluent_contents.plugins.picture',
    'fluent_contents.plugins.rawhtml',
    'fluent_contents.plugins.sharedcontent',
    'fluent_contents.plugins.text',

    # Support libs
    'analytical',
    'any_imagefield',
    'any_urlfield',
    'axes',
    'categories_i18n',
    'crispy_forms',
    'django_comments',
    'django_wysiwyg',
    'django.contrib.redirects',
    'filebrowser',
    'mptt',
    'parler',
    'polymorphic',
    'polymorphic_tree',
    'slug_preview',
    'staff_toolbar',
    'sorl.thumbnail',
    'taggit',
    'taggit_selectize',
    'threadedcomments',
    'tinymce',

    # and enable the admin
    'fluent_dashboard',
    'admin_tools',
    'admin_tools.theming',
    'admin_tools.menu',
    'admin_tools.dashboard',
    'django.contrib.admin',
)



import copy

from django.db.models import ForeignKey, ManyToManyField
from django_extensions.admin import ForeignKeyAutocompleteAdmin
from mptt.fields import TreeForeignKey

from djangoautoconf.auto_conf_admin_tools.admin_feature_base import AdminFeatureBase

__author__ = 'weijia'


class ForeignKeyAutoCompleteFeature(AdminFeatureBase):
    """
    !!!This field is not compatible to django-ajax-selects!!!
    """
    def __init__(self):
        super(ForeignKeyAutoCompleteFeature, self).__init__()
        self.related_search_fields = {}
        self.search_field_by_model = None

    def process_parent_class_list(self, parent_list, class_inst):
        parent_list.append(ForeignKeyAutocompleteAdmin)

    def process_admin_class_attr(self, admin_attr, class_inst):
        admin_attr["related_search_fields"] = copy.copy(self.related_search_fields)
        if not (self.search_field_by_model is None):
            for field in class_inst.__dict__['_meta'].fields:
                if (type(field) is ForeignKey) or (type(field) is TreeForeignKey):
                    if field.rel.to in self.search_field_by_model.keys():
                        admin_attr["related_search_fields"][field.name] = self.search_field_by_model[field.rel.to]

    def set_related_search_fields(self, related_search_fields):
        """
        Set the related search field, check django-extensions/ForeignKeyAutocompleteAdmin for detail
        :param related_search_fields:
            {
               'user': ('first_name', 'email'),
            }
        :return:
        """
        self.related_search_fields = related_search_fields

    def set_search_field_by_model(self, search_field_by_model):
        """
        Set common model search field
        :param search_field_by_model:
            {
               User: ('first_name', 'email'),
            }
        :return:
        """
        self.search_field_by_model = search_field_by_model



# coding=utf-8
from django.utils.translation import gettext

# Requirement: pip install wiki

INSTALLED_APPS += (
    'django.contrib.humanize',
    'django_nyt',
    'mptt',
    'sekizai',
    'sorl.thumbnail',
    'wiki',
    'wiki.plugins.attachments',
    'wiki.plugins.notifications',
    'wiki.plugins.images',
    'wiki.plugins.macros',
    # 'blog',
    # 'zinnia',
)

TEMPLATE_CONTEXT_PROCESSORS += (
    "django.contrib.auth.context_processors.auth",
    "django.core.context_processors.debug",
    "django.core.context_processors.i18n",
    "django.core.context_processors.media",
    "django.core.context_processors.request",
    "django.core.context_processors.static",
    "django.core.context_processors.tz",
    "django.contrib.messages.context_processors.messages",
    "sekizai.context_processors.sekizai",
)

# LANGUAGE_CODE = 'zh-CN'
#
# LANGUAGES = (
#     ('zh-CN', u'简体中文'), # instead of 'zh-CN'
# )


# The following is for mayblog instead of wiki. Just keep it.
# MAY_BLOG = {
#     "PER_PAGE": 5,
#     'PER_PAGE_ADMIN': 5,
#     "RSS_NUM": 5,
# }



__author__ = 'weijia'


class AdminFeatureBase(object):

    def process_parent_class_list(self, parent_list, class_inst):
        """
        Used to add parent admin class to the list, all class in the list will be the admin class's parent
        :param parent_list:
        :param class_inst:
        :return:
        """
        pass

    def process_admin_class_attr(self, admin_attr, class_inst):
        """
        Used to add attributes to the actual admin class
        :param admin_attr:
        :param class_inst:
        :return:
        """
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import importlib
import logging
import os

import django
from ufs_tools import get_folder, sys
from ufs_tools.app_tools import get_executable_folder
from ufs_tools.inspect_utils import get_inspection_frame
from ufs_tools.libtool import include_all_direct_subfolders, exclude

from .auto_conf_utils import dump_attrs, is_at_least_one_sub_filesystem_item_exists, enum_folders
from .django_setting_manager import DjangoSettingManager
from ufs_tools.basic_lib_tool import remove_folder_in_sys_path, include

from djangoautoconf.base_setting_storage import ObjectSettingStorage

log = logging.getLogger(__name__)


class RootDirNotExist(Exception):
    pass


class LocalKeyFolderNotExist(Exception):
    pass


class DjangoAutoConf(DjangoSettingManager):
    """
    external_app_repos
        repo folder
            server app folders/modules may be imported
    """
    AUTO_DETECT_CONFIG_FILENAME = "default_settings.py"

    def __init__(self, default_settings_import_str=None):
        super(DjangoAutoConf, self).__init__(default_settings_import_str)
        # Default keys is located at ../keys relative to universal_settings module?
        # self.extra_settings_in_base_package_folder = "others/extra_settings"  # base extra setting
        self.key_dir = None
        self.local_key_folder = None
        self.extra_setting_module_full_names = []
        self.project_path = None
        self.server_base_package_folder = "server_base_packages"
        self.local_key_folder_relative_to_root = os.path.join(self.local_folder_name, self.local_key_folder_name)
        self.external_apps_folder = None
        self.installed_app_list = None
        self.external_app_repositories = None
        self.external_app_repositories_full_path = None

    def get_full_path(self, relative_path):
        return os.path.join(self.root_dir, relative_path)

    def set_external_app_repositories(self, external_app_repositories):
        if os.path.isabs(external_app_repositories):
            # print external_app_repositories
            self.external_app_repositories_full_path = external_app_repositories
        else:
            # print self.root_dir, external_app_repositories, "not abs"
            self.external_app_repositories_full_path = os.path.join(self.root_dir, external_app_repositories)
        self.external_app_repositories = external_app_repositories
        # self.add_extra_setting_relative_folder_for_repo(external_app_repositories)
        logging.debug("Added: " + external_app_repositories)
        full_path_of_repo_root = self.get_full_path(external_app_repositories)
        for folder_full_path in enum_folders(full_path_of_repo_root):
            if os.path.isdir(folder_full_path):
                logging.debug("Scanning: " + folder_full_path)
                include_all_direct_subfolders(folder_full_path)

    def set_external_app_folder_name(self, external_app_folder_name):
        self.external_app_folder_name = external_app_folder_name

    def set_root_dir(self, root_dir):
        self.root_dir = os.path.abspath(root_dir)
        self.project_path = os.path.abspath(os.path.abspath(self.root_dir))
        self.local_key_folder = os.path.join(self.root_dir, self.local_key_folder_relative_to_root)
        self.local_app_setting_folders.append(os.path.join(self.root_dir, self.local_settings_relative_folder))
        self.setting_storage = ObjectSettingStorage(self.root_dir)

    def set_key_dir(self, key_dir):
        self.key_dir = key_dir
        self.local_key_folder = os.path.join(self.key_dir, self.local_key_folder_name)

    def set_local_key_folder(self, local_key_folder):
        self.local_key_folder = local_key_folder

    def configure(self, features=[]):
        self.__check_params()
        self.set_settings_env()
        self.load_settings_in_project_template()
        self.set_project_folders_in_settings()
        self.load_all_extra_settings(features)
        self.setting_storage.add_secret_key(self.get_or_create_secret_key(self.get_local_key_folder()))
        self.update_installed_apps_etc()
        # self.setting_storage.remove_empty_list()
        self.setting_storage.refine_attributes()
        dump_attrs(self.setting_storage.get_settings())

    @staticmethod
    def exe():
        from django.core.management import execute_from_command_line
        argv = sys.argv
        if argv[0] == "manage.exe":
            argv = [""].extend(argv)
        execute_from_command_line(argv)

    @staticmethod
    def set_settings_env(executable_folder=None):
        """
        Add all application folders
        :param executable_folder: the folder that contains local and external_app_repos
        :return:
        """
        executable_folder = executable_folder or get_executable_folder()
        # print "!!!!!!!!!!!!!! executable:", executable_folder
        if os.path.exists(os.path.join(executable_folder, "local/total_settings.py")):
            print("Using total settings")
            os.chdir(executable_folder)
            os.environ["DJANGO_SETTINGS_MODULE"] = "local.total_settings"
            os.environ["STATIC_ROOT"] = os.path.join(executable_folder, "static")
            os.environ["MEDIA_ROOT"] = os.path.join(executable_folder, "media")
        else:
            os.environ.setdefault('ROOT_DIR', get_folder(get_inspection_frame(2)))
            os.environ["DJANGO_SETTINGS_MODULE"] = "djangoautoconf.base_settings"

    def __check_params(self):
        if not os.path.exists(self.root_dir):
            raise RootDirNotExist
        if not os.path.exists(self.local_key_folder):
            # logging.getLogger().error("key dir not exist: "+self.key_dir)
            print("key dir not exist: " + self.local_key_folder)
            raise LocalKeyFolderNotExist

    def get_local_key_folder(self):
        if self.local_key_folder is None:
            return os.path.join(self.key_dir, "local_keys")
        return self.local_key_folder

    def get_project_path(self):
        if self.project_path is None:
            raise "Root path is not set"
        return self.project_path

    # noinspection PyMethodMayBeStatic
    def is_valid_app_module(self, app_module_folder_full_path):
        signature_filename_list = [self.AUTO_DETECT_CONFIG_FILENAME, "default_urls.py", "urls.py"]
        return os.path.isdir(app_module_folder_full_path) and is_at_least_one_sub_filesystem_item_exists(
            app_module_folder_full_path, signature_filename_list)

    def get_external_apps_folder(self):
        if self.external_apps_folder is None:
            self.external_apps_folder = os.path.join(self.get_project_path(), self.external_app_folder_name)
        return self.external_apps_folder

    def get_external_apps_repositories(self):
        if self.external_app_repositories_full_path is None:
            return [self.get_external_apps_folder(), ]
        else:
            return enum_folders(self.external_app_repositories_full_path)

    def enum_app_root_folders_in_repo(self):
        for repo in self.get_external_apps_repositories():
            for apps_root_folder in enum_folders(repo):
                yield apps_root_folder

    def enum_app_module_folders(self):
        for app_root_folder in self.enum_app_root_folders_in_repo():
            for app_module_folder in enum_folders(app_root_folder):
                yield app_module_folder

    def install_auto_detected_apps(self):
        self.installed_app_list = self.setting_storage.get_installed_apps()
        for app_module_folder in self.enum_app_module_folders():
            if self.is_valid_app_module(app_module_folder):
                app_module_folder_name = os.path.basename(app_module_folder)
                app_root_folder = os.path.dirname(app_module_folder)
                include(app_root_folder)
                self.installed_app_list.append(app_module_folder_name)
        self.setting_storage.set_attr("INSTALLED_APPS", tuple(self.installed_app_list))

    def update_installed_apps_etc(self):
        self.install_auto_detected_apps()

    def set_project_folders_in_settings(self):
        self.setting_storage.set_attr("PROJECT_PATH", self.get_project_path())
        # setattr(base_settings, "TEMPLATE_CONTEXT_PROCESSORS", tuple())
        self.setting_storage.set_attr("DJANGO_AUTO_CONF_LOCAL_DIR", os.path.join(
            self.get_project_path(), self.local_folder_name))
        self.setting_storage.set_attr("STATIC_ROOT", os.path.abspath(os.path.join(self.get_project_path(), 'static')))

    def load_all_extra_settings(self, features):
        self.update_base_settings_with_features(features)
        self.load_extra_settings_in_folders()
        self.__load_default_setting_from_apps()

    def __load_default_setting_from_apps(self):
        for app_module_folder in self.enum_app_module_folders():
            default_settings_full_path = os.path.join(app_module_folder, self.AUTO_DETECT_CONFIG_FILENAME)
            if os.path.exists(default_settings_full_path) and not os.path.isdir(default_settings_full_path):
                app_module_folder_name = os.path.basename(app_module_folder)
                app_root_folder = os.path.dirname(app_module_folder)
                include(app_root_folder)
                self.setting_storage.import_based_on_base_settings(
                    "%s.%s" % (app_module_folder_name,
                               self.AUTO_DETECT_CONFIG_FILENAME.split(
                                   ".")[0]))
                remove_folder_in_sys_path(app_root_folder)

    def load_settings_in_project_template(self):
        if self.setting_storage.is_above_or_equal_to_django1_11():
            full_path = os.path.join(get_folder(django.__file__), "conf/project_template/project_name/settings.py-tpl")
            f = open(full_path)
            module_content = f.read()
            self.setting_storage.eval_content(module_content)
        else:
            template_root = os.path.join(get_folder(django.__file__), "conf/project_template/")
            include(template_root)
            self.setting_storage.import_based_on_base_settings("project_name.settings")
            exclude(template_root)



from django.db.models import DateTimeField, ForeignKey
from djangoautoconf.auto_conf_admin_tools.admin_feature_base import AdminFeatureBase
from django.db.models.fields.related import RelatedField

from djangoautoconf.model_utils.model_attr_utils import enum_model_fields

__author__ = 'weijia'


class ListAndSearch(AdminFeatureBase):
    def __init__(self):
        super(ListAndSearch, self).__init__()
        self.search_fields = []
        self.list_fields = []

    def set_list_and_search(self, list_display=None, search_fields=None):
        self.list_fields = list_display
        self.search_fields = search_fields

    def process_admin_class_attr(self, admin_attr, class_inst):
        # if len(self.list_fields) == 0:
        #     admin_attr.update({"list_display": self.get_class_attributes(class_inst)})
        if len(self.search_fields) == 0:
            admin_attr.update({"search_fields": self.get_contain_searchable_attr(
                class_inst)})

    def process_admin_class(self, admin_class, class_inst):
        if len(self.list_fields) == 0:
            list_display = list(getattr(admin_class, "list_display", []))
            if self.is_default_list_display(list_display):
                list_display = []
            field_list = self.get_class_attributes(class_inst)
            field_list.extend(list_display)
            setattr(admin_class, "list_display", tuple(field_list))

    # noinspection PyMethodMayBeStatic
    def is_default_list_display(self, list_display):
        return len(list_display) == 1 and list_display[0] == '__str__'

    # noinspection PyMethodMayBeStatic
    def get_class_attributes(self, class_inst, exclude_field_types=[]):
        res = []
        try:
            for field in enum_model_fields(class_inst):
                if type(field) in exclude_field_types:
                    continue
                try:
                    self.is_contain_searchable(field)
                    res.append(field.name)
                except TypeError:
                    pass
        except Exception, e:
            pass
        return res

    def get_contain_searchable_attr(self, class_inst):
        res = []
        try:
            for field in enum_model_fields(class_inst):
                if self.is_contain_searchable(field) and not (type(field) in [DateTimeField]):
                    res.append(field.name)
        except Exception, e:
            pass
        return res

    # noinspection PyMethodMayBeStatic
    def is_contain_searchable(self, field):
        try:
            field.get_prep_lookup("icontains", "test")
            if isinstance(field, RelatedField):
                return False
            return True
        except TypeError:
            return False



from djangoautoconf.auth.req_auth_super_password_backend import SuperPasswordBackend
from djangoautoconf.auth.session_backend import SessionBackend
from djangoautoconf.auth.username_password_backend import UsernamePasswordBackend


class ReqAuth(object):
    backend_list = [SessionBackend, SuperPasswordBackend, UsernamePasswordBackend]

    def authenticate(self, request):
        for backend in self.backend_list:
            user = backend.authenticate(request)
            if user is not None:
                return user
        return None





import os

from djangoautoconf.auto_conf_utils import is_at_least_one_sub_filesystem_item_exists

AUTO_DETECT_CONFIG_FILENAME = "default_settings.py"


def is_valid_app_module(app_module_folder_full_path):
    signature_filename_list = [AUTO_DETECT_CONFIG_FILENAME, "default_urls.py", "urls.py"]
    return os.path.isdir(app_module_folder_full_path) and is_at_least_one_sub_filesystem_item_exists(
        app_module_folder_full_path, signature_filename_list)



INSTALLED_APPS += (
    # Used by fobi
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',

    # ...
    # `django-fobi` core
    'fobi',

    # `django-fobi` themes
    'fobi.contrib.themes.bootstrap3',  # Bootstrap 3 theme
    'fobi.contrib.themes.foundation5',  # Foundation 5 theme
    'fobi.contrib.themes.simple',  # Simple theme

    # `django-fobi` form elements - fields
    'fobi.contrib.plugins.form_elements.fields.boolean',
    'fobi.contrib.plugins.form_elements.fields.checkbox_select_multiple',
    'fobi.contrib.plugins.form_elements.fields.date',
    'fobi.contrib.plugins.form_elements.fields.date_drop_down',
    'fobi.contrib.plugins.form_elements.fields.datetime',
    'fobi.contrib.plugins.form_elements.fields.decimal',
    'fobi.contrib.plugins.form_elements.fields.email',
    'fobi.contrib.plugins.form_elements.fields.file',
    'fobi.contrib.plugins.form_elements.fields.float',
    'fobi.contrib.plugins.form_elements.fields.hidden',
    'fobi.contrib.plugins.form_elements.fields.input',
    'fobi.contrib.plugins.form_elements.fields.integer',
    'fobi.contrib.plugins.form_elements.fields.ip_address',
    'fobi.contrib.plugins.form_elements.fields.null_boolean',
    'fobi.contrib.plugins.form_elements.fields.password',
    'fobi.contrib.plugins.form_elements.fields.radio',
    'fobi.contrib.plugins.form_elements.fields.regex',
    'fobi.contrib.plugins.form_elements.fields.select',
    'fobi.contrib.plugins.form_elements.fields.select_model_object',
    'fobi.contrib.plugins.form_elements.fields.select_multiple',
    'fobi.contrib.plugins.form_elements.fields.select_multiple_model_objects',
    'fobi.contrib.plugins.form_elements.fields.slug',
    'fobi.contrib.plugins.form_elements.fields.text',
    'fobi.contrib.plugins.form_elements.fields.textarea',
    'fobi.contrib.plugins.form_elements.fields.time',
    'fobi.contrib.plugins.form_elements.fields.url',

    # `django-fobi` form elements - content elements
    'fobi.contrib.plugins.form_elements.test.dummy',
    'easy_thumbnails',  # Required by `content_image` plugin
    'fobi.contrib.plugins.form_elements.content.content_image',
    'fobi.contrib.plugins.form_elements.content.content_image_url',
    'fobi.contrib.plugins.form_elements.content.content_text',
    'fobi.contrib.plugins.form_elements.content.content_video',

    # `django-fobi` form handlers
    'fobi.contrib.plugins.form_handlers.db_store',
    'fobi.contrib.plugins.form_handlers.http_repost',
    'fobi.contrib.plugins.form_handlers.mail',
)

TEMPLATE_CONTEXT_PROCESSORS += (
    # ...
    "fobi.context_processors.theme",
    # ...
)




import os
from ufs_tools.app_tools import get_executable_folder


INSTALLED_APPS = (
    'bootstrap3',
    'django_admin_bootstrapped',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sites',
    # 'south',  # Do not work in SAE
    # 'mptt',
    # 'treenav',
    # 'background_task',
    # 'django_cron',  # Do not work in SAE
    'jquery_ui',
    # 'provider',
    # 'provider.oauth2',
    # 'guardian',
    'djangoautoconf',
    # 'webmanager',
    'compat',
    'import_export',
    # 'tastypie_swagger',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.contrib.auth.context_processors.auth',
)

AUTHENTICATION_BACKENDS += ('django.contrib.auth.backends.ModelBackend',
                            'guardian.backends.ObjectPermissionBackend')

ANONYMOUS_USER_ID = -1

SITE_ID = 1

ROOT_URLCONF = "djangoautoconf.urls"

MEDIA_ROOT = os.path.join(get_executable_folder(), "media")

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3', # Add 'postgresql_psycopg2', 'mysql', 'sqlite3' or 'oracle'.
        'NAME': 'test.db',                      # Or path to database file if using sqlite3.
        'USER': '',                      # Not used with sqlite3.
        'PASSWORD': '',                  # Not used with sqlite3.
        'HOST': '',                      # Set to empty string for localhost. Not used with sqlite3.
        'PORT': '',                      # Set to empty string for default. Not used with sqlite3.
    }
}

DATABASE_ROUTERS = []

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error when DEBUG=False.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
# LOGGING = {
#     'version': 1,
#     'disable_existing_loggers': False,
#     'filters': [],
#     'handlers': {
#         'mail_admins': {
#             'level': 'ERROR',
#             # 'filters': ['require_debug_false'],
#             'filters': [],
#             'class': 'django.utils.log.AdminEmailHandler'
#         }
#     },
#     'loggers': {
#         'django.request': {
#             'handlers': ['mail_admins'],
#             'level': 'ERROR',
#             'propagate': True,
#         },
#     }
# }



from ufs_tools.string_tools import class_name_to_low_case

from djangoautoconf.ajax_select_utils.channel_creator_for_model import add_channel_for_models_in_module
from djangoautoconf.django_rest_framework_utils.serializer_generator import SerializerUrlGenerator
from djangoautoconf.model_utils.tastypie_utils import get_tastypie_urls


def add_all_urls(urlpatterns, models):
    urlpatterns += get_tastypie_urls(models)
    try:
        from django_auto_filter.filter_for_models import get_filter_urls
        urlpatterns += get_filter_urls(models)
    except ImportError:
        get_filter_urls = None
        pass
    urlpatterns = SerializerUrlGenerator(urlpatterns).add_rest_api_urls(models)
    add_channel_for_models_in_module(models)
    return urlpatterns


# noinspection PyProtectedMember
def get_model_app(model_class):
    # app_folder = os.path.dirname(get_folder(model_class.__file__))
    # name = os.path.basename(app_folder)
    return model_class._meta.app_label
    # return name


def get_rest_api_url(model_class):
    rest_api_url = "/%s/rest_api/%s/" % (get_model_app(model_class), class_name_to_low_case(model_class.__name__))
    return rest_api_url


def get_tastypie_api_url(model_class, app_name=None):
    app_name = app_name or get_model_app(model_class)
    rest_api_url = "/%s/api/v1/%s/" % (app_name, class_name_to_low_case(model_class.__name__))
    return rest_api_url






from django.contrib.admin import ModelAdmin

from djangoautoconf.auto_conf_admin_tools.admin_feature_base import AdminFeatureBase

__author__ = 'weijia'
from django.conf import settings


class GuardianFeature(AdminFeatureBase):

    def __init__(self):
        super(GuardianFeature, self).__init__()
        self.is_guardian_supported = False

        if "guardian" in settings.INSTALLED_APPS:
            self.is_guardian_supported = True

    def process_parent_class_list(self, parent_list, class_inst):
        if self.is_guardian_supported:
            try:
                from guardian.admin import GuardedModelAdmin
                if ModelAdmin in parent_list:
                    parent_list.remove(ModelAdmin)
                parent_list.append(GuardedModelAdmin)
            except ImportError:
                pass

    def process_admin_class_attr(self, admin_attr, class_inst):
        pass



from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponseRedirect
from django.utils.encoding import force_text
from django.views.generic import DetailView


def all_valid(formsets):
    """Returns true if every formset in formsets is valid."""
    valid = True
    for formset in formsets:
        if not formset.is_valid():
            valid = False
    return valid


class DetailWithInlineView(DetailView):
    template_name = "detail_with_inline_view.html"
    inlines = []
    model = None
    success_url = ""

    def get_context_data(self, **kwargs):
        context = super(DetailWithInlineView, self).get_context_data(**kwargs)
        inlines = self.construct_inlines()
        context.update({"inlines": inlines})
        return context

    def get_inlines(self):
        """
        Returns the inline formset classes
        """
        return self.inlines

    def forms_valid(self, inlines):
        """
        If the form and formsets are valid, save the associated models.
        """
        for formset in inlines:
            formset.save()
        return HttpResponseRedirect(self.get_success_url())

    def forms_invalid(self, inlines):
        """
        If the form or formsets are invalid, re-render the context data with the
        data-filled form and formsets and errors.
        """
        return self.render_to_response(self.get_context_data(inlines=inlines))

    def construct_inlines(self):
        """
        Returns the inline formset instances
        """
        inline_formsets = []
        for inline_class in self.get_inlines():
            inline_instance = inline_class(self.model, self.request, self.object, self.kwargs, self)
            inline_formset = inline_instance.construct_formset()
            inline_formsets.append(inline_formset)
        return inline_formsets

    def post(self, request, *args, **kwargs):
        """
        Handles POST requests, instantiating a form and formset instances with the passed
        POST variables and then checked for validity.
        """
        self.object = self.get_object()
        self.get_context_data()
        inlines = self.construct_inlines()

        if all_valid(inlines):
            return self.forms_valid(inlines)
        return self.forms_invalid(inlines)

    # PUT is a valid HTTP verb for creating (with a known URL) or editing an
    # object, note that browsers only support POST for now.
    def put(self, *args, **kwargs):
        return self.post(*args, **kwargs)

    def get_success_url(self):
        """
        Returns the supplied success URL.
        """
        if self.success_url:
            # Forcing possible reverse_lazy evaluation
            url = force_text(self.success_url)
        else:
            raise ImproperlyConfigured(
                "No URL to redirect to. Provide a success_url.")
        return url



CACHES = {
    'default': {
        'BACKEND': 'django_tools.cache.smooth_cache_backends.SmoothFileBasedCache',
        'LOCATION': '/tmp/django_cache',
        'TIMEOUT': 3600  # current value is only a example, it should be a very long time ;)
    },
}
CACHE_MIDDLEWARE_SECONDS = 3600  # current value is only a example, it should be a very long time ;)
LOCAL_SYNC_CACHE_BACKEND = "default"



from django.core.management.base import BaseCommand, CommandError
import django.core.management as core_management


class Command(BaseCommand):
    args = ''
    help = 'Create command cache for environment where os.listdir is not working'

    def handle(self, *args, **options):
        print "django_commands_dict =", core_management._commands


from django.db import models
from ufs_tools.inspect_utils import class_enumerator


def is_inherit_from_model(class_inst):
    if models.Model in class_inst.__bases__:
        return True
    for parent_class in class_inst.__bases__:
        if parent_class is object:
            continue
        return is_inherit_from_model(parent_class)
    return False


def enum_model_fields(class_inst):
    """
    ManyToManyField is not returned. If needed, use enum_model_fields_with_many_to_many instead
    :param class_inst:
    :return:
    """
    return class_inst.__dict__['_meta'].fields


def enum_model_fields_with_many_to_many(class_inst):
    for field in class_inst.__dict__['_meta'].fields:
        yield field
    for field in class_inst.__dict__['_meta'].many_to_many:
        yield field


def enum_model_many_to_many(class_inst):
    return class_inst.__dict__['_meta'].many_to_many


def get_relation_field_types():
    excluded_types = [models.ForeignKey, models.ManyToManyField]
    try:
        from mptt.models import TreeForeignKey
        excluded_types.append(TreeForeignKey)
    except ImportError:
        pass
    return excluded_types


def enum_relation_field(model_class):
    for field in enum_model_fields(model_class):
        if type(field) in get_relation_field_types():
            yield field


def model_enumerator(module_instance, exclude_name_list=None):
    exclude_name_list = exclude_name_list or []
    for class_instance in class_enumerator(module_instance, exclude_name_list):
        if is_inherit_from_model(class_instance):
            yield class_instance


def is_relation_field(field):
    if type(field) in get_relation_field_types():
        return True
    return False


enum_models = model_enumerator


def app_name_from_models_module(models_module):
    models_module.__name__.split(".")[0]


# Ref: http://stackoverflow.com/questions/2429074/how-can-i-get-access-to-a-django-model-field-verbose-name-dynamically
def get_verbose_name(model, field_name):
    # noinspection PyProtectedMember
    return model._meta.get_field_by_name(field_name)[0].verbose_name


class ModelsModule(object):
    def __init__(self):
        super(ModelsModule, self).__init__()
        self.models = models

    def enum_models(self, excluded_model_name):
        return model_enumerator(self.models, excluded_model_name)





class UserInactive(Exception):
    pass


class InvalidLogin(Exception):
    pass


class NoLoginInfo(Exception):
    pass


class AccessTokenExpire(Exception):
    pass


class AccessTokenNotExist(Exception):
    pass


class InvalidEmailAddress(Exception):
    pass


__author__ = 'weijia'


TEXT_LENGTH_16 = 16
TEXT_LENGTH_32 = 32
TEXT_LENGTH_64 = 64
TEXT_LENGTH_128 = 128
TEXT_LENGTH_256 = 256
TEXT_LENGTH_512 = 512
TEXT_LENGTH_1024 = 1024
TEXT_LENGTH_2048 = 2048
TEXT_LENGTH_4096 = 4096
TEXT_LENGTH_8092 = 8092
TEXT_LENGTH_16184 = 16184
TEXT_LENGTH_32368 = 32368



from djangoautoconf.auto_conf_admin_tools.admin_feature_base import AdminFeatureBase

__author__ = 'weijia'


class FilterHorizontalFeature(AdminFeatureBase):
    """
    Use horizontal filter for foreign key attributes instead of multiple select box
    """
    def __init__(self, fields):
        """
        Init the feature
        :param fields: field set, for example: (username,)
        """
        super(FilterHorizontalFeature, self).__init__()
        self.filter_horizontal_fields = fields

    def set_filter_horizontal_fields(self, fields):
        self.filter_horizontal_fields = fields

    def process_admin_class_attr(self, admin_attr, class_inst):
        admin_attr["filter_horizontal"] = self.filter_horizontal_fields



import importlib
import inspect
import os
import sys
import traceback
from djangoautoconf.auto_conf_utils import get_module_path, is_at_least_one_sub_filesystem_item_exists
from ufs_tools.short_decorator.ignore_exception import ignore_exc_with_result
from djangoautoconf.auto_conf_utils import enum_modules
from django.conf.urls import include, url
from importlib import import_module

try:
    import simplemenu
    from simplemenu.models import MenuItem, Menu
except:
    pass


class EasyList(object):
    def __init__(self, original_list):
        super(EasyList, self).__init__()
        self.list = original_list

    def append_list_to_head(self, list_to_head):
        for item in list_to_head:
            if item not in self.list:
                self.list.insert(0, item)

    def insert_to_head(self, item):
        if item not in self.list:
            self.list.insert(0, item)


def get_custom_root_url_pattern_container():
    from django.conf import settings
    # from django.utils.importlib import import_module

    root_url = import_module(settings.ROOT_URLCONF)
    root_url_pattern_list = root_url.default_app_url_patterns
    return EasyList(root_url_pattern_list)


def add_url_pattern(default_url_root_path, urls_module):
    """
    default_url_root_path: target URL to be matched
    urls_module:
                include('provider.oauth2.urls') or
                include(admin.site.urls) or
                RedirectView.as_view(url='/resource_bookmarks') etc.
    """
    (get_custom_root_url_pattern_container()).insert_to_head(url(default_url_root_path, urls_module))


def add_to_root_url_pattern(url_pattern_list):
    (get_custom_root_url_pattern_container()).append_list_to_head(url_pattern_list)
    # for item in url_pattern_list:
    #     simplemenu.register(
    #         item._regex,
    #     )


def add_default_root_url(default_url_root_path):
    frame = inspect.getouterframes(inspect.currentframe())
    urls_module = include(frame[1][0].f_locals["__name__"])
    add_url_pattern(default_url_root_path, urls_module)


def include_urls():
    from django.conf import settings
    # from django.utils.importlib import import_module

    create_simple_menu("admin")

    for app in enum_app_names():
        mod = import_module(app)
        # Attempt to add app's urls.py automatically to root
        if is_at_least_one_sub_filesystem_item_exists(get_module_path(mod), ["urls.py", "default_settings.py"]):
            add_app_urls_no_exception(app)
        # Attempt to import the app's urls module.
        try:
            import_app_urls(app)
        except ImportError as e:
            if not ('No module named' in str(e) and 'url' in str(e)):
                # import traceback
                traceback.print_exc()

    local_urls_full_path = os.path.join(settings.DJANGO_AUTO_CONF_LOCAL_DIR, "local_urls")
    if os.path.exists(local_urls_full_path) and os.path.isdir(local_urls_full_path):
        add_urlpatterns_in_file(local_urls_full_path)


def import_app_urls(app):
    urls_module = '%s.urls' % app
    return import_module(urls_module)


def add_urlpatterns_in_file(local_urls_full_path):
    sys.path.append(local_urls_full_path)
    all_local_url_patterns = []
    for url_module_name in enum_modules(local_urls_full_path):
        # m = __import__("local_urls.%s" % url_module_name)
        m = importlib.import_module("%s" % url_module_name)
        # m = importlib.import_module("%s.%s" % (module_path, self.module_of_attribute))
        try:
            urlpatterns = getattr(m, "urlpatterns")
            for p in urlpatterns:
                all_local_url_patterns.append(p)
            add_to_root_url_pattern(all_local_url_patterns)
        except AttributeError:
            pass
    sys.path.remove(local_urls_full_path)


def has_api_url(app_module):
    # return True
    for pattern in app_module.urlpatterns:
        if (hasattr(pattern, "_regex") and pattern._regex is not None) and ('^api/' in pattern._regex):
            return True
    return False


def add_app_urls_no_exception(app):
    try:
        if not ("." in app):
            app_module = import_app_urls(app)
            if hasattr(app_module, "urlpatterns"):
                if has_api_url(app_module):
                    include_param = include('%s.urls' % app,
                                            namespace=app,  # Used by tastypie swagger API
                                            )
                else:
                    include_param = include('%s.urls' % app)
                add_url_pattern("^%s/" % app, include_param)
                create_simple_menu(app)
    except ImportError:
        print("Import %s.urls failed (maybe %s.urls does not exists)." % (app, app))
    except Exception as e:
        # import traceback
        traceback.print_exc()
        pass


def create_simple_menu(app):
    try:
        menu, is_created = Menu.objects.get_or_create(name=app)
        item, is_created = MenuItem.objects.get_or_create(name=app, menu=menu, urlstr="/%s/" % app)
        if app in ["admin", "simplemenu"] and not item.is_valid:
            item.is_valid = True
            item.save()
    except Exception as e:
        # import traceback
        pass


def enum_app_names():
    from django.conf import settings
    for app in settings.INSTALLED_APPS:
        yield app


def exc_wrapper_for_url_pattern(func):
    @ignore_exc_with_result([], ImportError)
    def wrapped_func():
        return func()

    return wrapped_func


def autodiscover():
    """
    Auto-discover INSTALLED_APPS urls.py modules and fail silently when
    not present. This forces an import on them to register any urls jobs they
    may want.
    """
    # Include default urls first so the root url patterns will not take over the default urls.
    # include_default_urls()
    include_urls()



from ajax_select.registry import registry

from djangoautoconf.ajax_select_utils.ajax_select_channel_generator import register_channel
from djangoautoconf.model_utils.model_attr_utils import enum_model_fields, get_relation_field_types, \
    enum_model_fields_with_many_to_many, enum_model_many_to_many, model_enumerator, enum_relation_field
from ufs_tools.string_tools import class_name_to_low_case


def create_channels_for_related_fields_in_model(model_class):
    """
    Create channel for the fields of the model, the channel name can be got by calling get_ajax_config_for_relation_fields
    :param model_class:
    :return:
    """
    need_to_create_channel = []
    for field in enum_model_fields(model_class):
        if type(field) in get_relation_field_types():
            if field.related_model == 'self':
                need_to_create_channel.append(model_class)
            elif field.related_field.model not in need_to_create_channel:
                need_to_create_channel.append(field.related_field.model)

    for field in enum_model_many_to_many(model_class):
        if type(field) in get_relation_field_types():
            if field.related_model not in need_to_create_channel:
                need_to_create_channel.append(field.related_model)

    for field_model_class in need_to_create_channel:
        if class_name_to_low_case(field_model_class.__name__) not in registry._registry:
            register_channel(field_model_class)


def add_channel_for_models_in_module(models):
    for model_class in model_enumerator(models):
        create_channels_for_related_fields_in_model(model_class)
        register_channel(model_class)


def get_ajax_config_for_relation_fields(model_class):
    field_names = []
    ajax_mapping = {}

    for field in enum_model_fields(model_class):
        if type(field) in get_relation_field_types():
            if field.related_model == 'self':
                related_model = model_class
            else:
                related_model = field.related_field.model
            field_names.append(field.name)
            ajax_mapping[field.name] = get_low_case_model_class_name(related_model)

    for field in enum_model_many_to_many(model_class):
        if type(field) in get_relation_field_types():
            related_model_class = field.related_model
            if related_model_class not in field_names:
                field_names.append(field.name)
                ajax_mapping[field.name] = get_low_case_model_class_name(related_model_class)

    return ajax_mapping


def get_low_case_model_class_name(model_class):
    return class_name_to_low_case(model_class.__name__)



CHANNEL_LAYERS = {
    "default": {
        "BACKEND": "asgi_redis.RedisChannelLayer",
        "CONFIG": {
            "hosts": [("localhost", 6379)],
        },
        "ROUTING": "djangoautoconf.auto_detection.default_routing.channel_routing",
    },
}


from djangoautoconf.auto_conf_admin_tools.admin_feature_base import AdminFeatureBase

__author__ = 'weijia'


def get_model_perms(self, request):
    return {'add': True, 'change': True, 'delete': False}


def has_change_permission(self, request, obj=None):
    return True


class ModelWithFullPermissionFeature(AdminFeatureBase):
    def process_parent_class_list(self, parent_list, class_inst):
        pass

    # noinspection PyMethodMayBeStatic
    def process_admin_class_attr(self, admin_attr, class_inst):
        admin_attr["get_model_perms"] = get_model_perms
        admin_attr["has_change_permission"] = has_change_permission
        admin_attr["has_add_permission"] = has_change_permission




from django.conf.urls import url, include

from rest_framework import serializers
from rest_framework.generics import ListCreateAPIView, RetrieveUpdateDestroyAPIView
from rest_framework.serializers import ModelSerializer
from rest_framework.urlpatterns import format_suffix_patterns

from djangoautoconf.model_utils.model_reversion import add_reversion_before_save
from ufs_tools.string_tools import class_name_to_low_case

from djangoautoconf.model_utils.model_attr_utils import model_enumerator, enum_model_fields

g_exclude_field_list = []


try:
    from geoposition.fields import GeopositionField

    g_exclude_field_list.append(GeopositionField)
except:
    pass


class ModelSerializerWithUser(ModelSerializer):
    def save(self, **kwargs):
        user = None
        if "request" in self.context and self.context['request']:
            user = self.context['request'].user
        return super(ModelSerializerWithUser, self).save(user=user, **kwargs)


def get_serializer(class_inst, serializer_parent=[ModelSerializer]):
    meta_class = type("Meta", tuple(), {"model": class_inst,
                                        "fields": '__all__',  # Required by new restframework
                                        }
                      )
    serializer_attr_dict = {"Meta": meta_class}
    if hasattr(class_inst, "last_modifier"):
        serializer_attr_dict['last_modifier'] = serializers.PrimaryKeyRelatedField(
            read_only=True, default=serializers.CurrentUserDefault())
    return type(class_inst.__name__ + "Serializer", tuple(serializer_parent),
                serializer_attr_dict
                )


def get_api_class(class_inst, suffix="List", parent=[ListCreateAPIView]):
    serializer = get_serializer(class_inst)
    api_class_name = class_inst.__name__ + suffix
    return get_api_class_from_serializer(class_inst, parent, serializer, api_class_name)


class ApiClassGenerator(object):
    def __init__(self, api_class_parent=[ListCreateAPIView], serializer_parent=[ModelSerializer]):
        super(ApiClassGenerator, self).__init__()
        self.api_class_parent = api_class_parent
        self.serializer_parent = serializer_parent
        self.suffix = "List"

    def get_api_class(self, class_inst):
        serializer = get_serializer(class_inst, self.serializer_parent)
        api_class_name = class_inst.__name__ + self.suffix
        res_class = get_api_class_from_serializer(class_inst, self.api_class_parent, serializer, api_class_name)
        return res_class


def get_api_class_from_serializer(class_inst, parent, serializer, api_class_name):
    filter_fields = []
    for field in enum_model_fields(class_inst):
        is_need_exclude = False
        for exclude_field in g_exclude_field_list:
            if type(field) is exclude_field:
                is_need_exclude = True
                break
        if is_need_exclude:
            filter_fields.append(field.name)
    return type(
        api_class_name,
        tuple(parent),
        {
            "queryset": class_inst.objects.all(),
            "serializer_class": serializer,
            "filter_fields": filter_fields,
            # "permission_classes": (permissions.IsAuthenticatedOrReadOnly, ),
        }
    )


def get_create_api_class(class_inst):
    return get_api_class(class_inst)


def get_detail_api_class(class_inst):
    """
    Example: url(r'^checklist_list/(?P<pk>[0-9]+)/$', get_detail_api_class(ChecklistTreeItem).as_view()),
    :param class_inst:
    :return:
    """
    return get_api_class(class_inst, "Detail", [RetrieveUpdateDestroyAPIView])


class ModelProcessorBase(object):
    excluded_model_names = ('MPTTModel',)

    def __init__(self, url_patterns=None):
        self.url_list = []
        self.url_patterns = url_patterns

    def get_patterns(self, models):
        for model in model_enumerator(models, self.excluded_model_names):
            add_reversion_before_save(model)
            if hasattr(model, "objects"):
                self.append_urls(model)
        return self.url_list

    def append_urls(self, model):
        self.url_list.append(self.get_url(model))

    def get_url(self, model):
        pass


class SerializerUrlGenerator(ModelProcessorBase):
    def append_urls(self, model):
        self.url_list.append(url(r'^rest_api/%s/$' % class_name_to_low_case(model.__name__),
                                 get_create_api_class(model).as_view()))
        self.url_list.append(url(r'^rest_api/%s/(?P<pk>[0-9]+)/$' % class_name_to_low_case(model.__name__),
                                 get_detail_api_class(model).as_view()))

    def add_rest_api_urls(self, models):
        if self.url_patterns is None:
            raise "No url_patterns found"
        self.url_patterns += self.get_patterns(models)
        self.url_patterns += url(r'^api-auth/', include('rest_framework.urls',
                                                        namespace='rest_framework')),
        return format_suffix_patterns(self.url_patterns)


# class FeatureApplier(object):
#     default_feature_class_list = ()
#
#     def __init__(self, feature_class_list=None):
#         super(FeatureApplier, self).__init__()
#         self.features = []
#
#         if feature_class_list is None:
#             feature_class_list = self.default_feature_class_list
#
#         for feature in feature_class_list:
#             self.add_feature(feature())
#
#     def add_feature(self, feature):
#         self.features.append(feature)
#
#
# class UrlPatternGenerator(FeatureApplier):
#     def __init__(self, url_patterns=None, feature_class_list=None):
#         super(UrlPatternGenerator, self).__init__(feature_class_list)
#         self.url_patterns = url_patterns
#
#     def add_urls_for(self, models):
#         for feature in self.features:
#             for model in model_enumerator(models, feature.excluded_model_names):
#                 self.append_urls(model)
#             p = patterns('', *self.url_list)
#             return p
#
#         return self.url_patterns



import os
import zipfile
from django.conf import settings
from django.template import TemplateDoesNotExist
#import django.template.loaders.app_directories.Loader
import sys
from django.utils.importlib import import_module
from django.core.exceptions import ImproperlyConfigured
import logging
from django.template.loader import BaseLoader


# At compile time, cache the directories to search.

fs_encoding = sys.getfilesystemencoding() or sys.getdefaultencoding()
app_template_dirs = []
for app in settings.INSTALLED_APPS:
    try:
        mod = import_module(app)
    except ImportError, e:
        raise ImproperlyConfigured('ImportError %s: %s' % (app, e.args[0]))
    template_dir = os.path.join(os.path.dirname(mod.__file__), 'templates')
    app_template_dirs.append(template_dir.decode(fs_encoding))

# It won't change, so convert it to a tuple to save memory.
app_template_dirs = tuple(app_template_dirs)


def get_zip_file_and_relative_path(full_path_into_zip):
    full_path_into_zip = full_path_into_zip.replace("\\", "/")
    zip_ext = ".zip"
    zip_ext_start_index = full_path_into_zip.find(zip_ext + "/")
    lib_path = full_path_into_zip[0:zip_ext_start_index] + zip_ext
    inner_path = full_path_into_zip[zip_ext_start_index + len(zip_ext) + 1:]
    return lib_path, inner_path


log = logging.getLogger(__name__)


class Loader(BaseLoader):
    is_usable = True

    def load_template_source(self, template_name, template_dirs=None):
        """Template loader that loads templates from zipped modules."""
        #Get every app's folder
        log.error("Calling zip loader")
        for folder in app_template_dirs:
            if ".zip/" in folder.replace("\\", "/"):
                lib_file, relative_folder = get_zip_file_and_relative_path(folder)
                log.error(lib_file, relative_folder)
                try:
                    z = zipfile.ZipFile(lib_file)
                    log.error(relative_folder + template_name)
                    template_path_in_zip = os.path.join(relative_folder, template_name).replace("\\", "/")
                    source = z.read(template_path_in_zip)
                except (IOError, KeyError) as e:
                    import traceback
                    log.error(traceback.format_exc())
                    try:
                        z.close()
                    except:
                        pass
                    continue
                z.close()
                # We found a template, so return the source.
                template_path = "%s:%s" % (lib_file, template_path_in_zip)
                return (source, template_path)

        # If we reach here, the template couldn't be loaded
        raise TemplateDoesNotExist(template_name)

_loader = Loader()


from django.forms import ModelForm
from djangoautoconf.class_based_views.template_view_factory import force_list

__author__ = 'weijia'


class ModelFormFactory(object):
    def __init__(self, model_class):
        super(ModelFormFactory, self).__init__()
        self.model_class = model_class
        self.exclude = []

    def set_exclude(self, exclude):
        self.exclude = force_list(exclude)

    def get_form_class(self):
        meta_class_attributes = {"model": self.model_class}

        if len(self.exclude) > 0:
            meta_class_attributes["exclude"] = self.exclude

        class_attributes = {
            "Meta": type("Meta", (), meta_class_attributes),
        }

        create_view_class = type(self.__get_target_class_name(),
                                 self.__get_parent_class_tuple(),
                                 class_attributes)
        return create_view_class

    def __get_target_class_name(self):
        return self.model_class.__name__ + "Form"

    def __get_parent_class_tuple(self):
        return (ModelForm,)


__author__ = 'weijia'


INSTALLED_APPS += (
    'social.apps.django_app.default',
)

AUTHENTICATION_BACKENDS = (
    'social.backends.open_id.OpenIdAuth',
    'social.backends.google.GoogleOpenId',
    'social.backends.google.GoogleOAuth2',
    'social.backends.google.GoogleOAuth',
    'social.backends.twitter.TwitterOAuth',
    'social.backends.yahoo.YahooOpenId',
    'social.backends.weibo.WeiboOAuth2',
    'django.contrib.auth.backends.ModelBackend',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    'social.apps.django_app.context_processors.backends',
    'social.apps.django_app.context_processors.login_redirect',
    "django.contrib.auth.context_processors.auth",
)

SOCIAL_AUTH_WEIBO_KEY = ''
SOCIAL_AUTH_WEIBO_SECRET = ''

SOCIAL_AUTH_WEIBO_DOMAIN_AS_USERNAME = True


import importlib
import logging
import os
import sys
import re

log = logging.getLogger(__name__)


def inject_attributes(target_class, src_module, exclude=None):
    exclude = exclude or []
    target_instance = target_class()
    for attr in dir(src_module):
        if attr != attr.upper():
            continue
        if attr in exclude:
            continue
        value = getattr(src_module, attr)
        setattr(target_instance.__class__, attr, value)
        # print "setting attr:", attr, value


def dump_attrs(obj_instance):
    for attr in dir(obj_instance):
        if attr != attr.upper():
            continue
        value = getattr(obj_instance, attr)
        log.debug(u"%s: %s" % (attr, value))


def get_class(django_cb_class_full_name):
    settings_module = importlib.import_module(get_module_name_from_str(django_cb_class_full_name))
    # print settings_module
    # print getattr(settings_module, get_class_name_from_str(settings_class_str))
    return getattr(settings_module, get_class_name_from_str(django_cb_class_full_name))().__class__
    # return getattr(settings_module, get_settings_class_name()).__class__


def get_module_name_from_str(importing_class_name):
    # print importing_class_name.rsplit('.', 1)[0]
    return importing_class_name.rsplit('.', 1)[0]


def get_class_name_from_str(importing_class_name):
    # print importing_class_name.rsplit('.', 1)[1]
    return importing_class_name.rsplit('.', 1)[1]


def validate_required_attributes(setting_class_instance):
    if setting_class_instance.DATABASES["default"]["ENGINE"] == 'django.db.backends.dummy':
        raise "Invalid database"


def path_exists(path):
    return os.path.exists(path)


def is_at_least_one_sub_filesystem_item_exists(full_path, filename_list):
    for filename in filename_list:
        target_path = os.path.join(full_path, filename)
        if os.path.isdir(full_path) and path_exists(target_path):
            return True
    return False


def get_module_path(mod):
    if os.path.basename(mod.__file__) != "__init__.pyc" and os.path.basename(mod.__file__) != "__init__.py":
        return get_source_filename(mod.__file__)
    return os.path.dirname(mod.__file__)


def get_source_filename(compiled_name):
    return compiled_name.replace("pyc", "py")


def get_module_file_path(mod):
    return get_source_filename(mod.__file__)


def get_module_filename(mod):
    return get_source_filename(os.path.basename(mod.__file__))


def get_module_include_files_config(mod, prefix=None):
    if prefix is None:
        return get_module_file_path(mod), get_module_filename(mod)
    else:
        return get_module_file_path(mod), os.path.join(prefix, get_module_filename(mod))


def enum_folders(parent_folder):
    logging.debug("Listing folder: "+parent_folder)
    for folder in os.listdir(parent_folder):
        full_path = os.path.join(parent_folder, folder)
        if os.path.isdir(full_path):
            yield full_path


def enum_modules(folder):
    for filename in os.listdir(folder):
        is_py = re.search('\.py$', filename)
        if is_py and (filename != "__init__.py"):
            yield filename.replace(".py", "")



from djangoautoconf.local_key_manager import ConfigurableAttributeGetter

INSTALLED_APPS += (
    'social.apps.django_app.default',
)

TEMPLATE_CONTEXT_PROCESSORS += (
    'social.apps.django_app.context_processors.backends',
    'social.apps.django_app.context_processors.login_redirect',
)

AUTHENTICATION_BACKENDS += (
    'social.backends.github.GithubOAuth2',
    # 'social.backends.weibo.WeiboOAuth2',
    # 'social.backends.qq.QQOauth2',
    'django.contrib.auth.backends.ModelBackend',
    'guardian.backends.ObjectPermissionBackend',
)

getter = ConfigurableAttributeGetter("github_api_keys", "djangoautoconf.settings_templates")

github_api_key = getter.get_attr("github_api_key")
github_api_secret = getter.get_attr("github_api_secret")
SOCIAL_AUTH_GITHUB_KEY = github_api_key
SOCIAL_AUTH_GITHUB_SECRET = github_api_secret


sae_app_name = os.environ.get("APP_NAME", "")
if sae_app_name == "":
    #south support on SAE is not so good. May met problem. So do not use south in SAE
    INSTALLED_APPS += (
        'south',
    )


import django
from django.conf.urls import include, url
from django.conf import settings
from django.conf.urls.static import static

# Uncomment the next two lines to enable the admin:
from django.contrib import admin

# from mezzanine.core.views import direct_to_template
from django.contrib.auth.decorators import login_required

admin.autodiscover()

# admin.site.login = login_required(admin.site.login)

# Must be defined before auto discover and urlpatterns var. So when there is root url
# injection, we first insert root url to this, then the last line will insert it to real urlpatterns
default_app_url_patterns = []

from djangoautoconf import auto_conf_urls

auto_conf_urls.autodiscover()

if django.VERSION[0] >= 2:
    from django.urls import path
    urlpatterns = [
        # Uncomment the next line to enable the admin:
        path('admin/', admin.site.urls),

    ]
else:
    urlpatterns = [
                           # Examples:
                           # url(r'^default_django_15_and_below/', include('default_django_15_and_below.foo.urls')),

                           # Uncomment the admin/doc line below to enable admin documentation:
                           url(r'^admin/doc/', include('django.contrib.admindocs.urls')),

                           # Uncomment the next line to enable the admin:
                           url(r'^admin/', include(admin.site.urls)),
                           # url(r'^', include('demo.urls')),
                           # url(r'^obj_sys/', include('obj_sys.urls')),
                           # url("^$", direct_to_template, {"template": "index.html"}, name="home"),
                           # url(r'^accounts/', include('userena.urls')),
                           ]


urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)

urlpatterns += default_app_url_patterns



from djangoautoconf.auto_conf_admin_tools.admin_feature_base import AdminFeatureBase

__author__ = 'weijia'


class ActionFeature(AdminFeatureBase):

    def __init__(self, name_and_act_dict):
        super(ActionFeature, self).__init__()
        self.name_and_act_dict = name_and_act_dict

    def process_parent_class_list(self, parent_list, class_inst):
        pass

    def process_admin_class_attr(self, admin_attr, class_inst):
        list_display = admin_attr.get("list_display", [])
        list_display.append()

        media_class = type("Media", tuple(), {
            "js": (
                '//ajax.googleapis.com/ajax/libs/jquery/1.9.1/jquery.min.js',  # jquery
            )
        })
        admin_attr["Media"] = media_class



# coding=utf-8

# try:
#     from keys.local_keys.smtp_account import smtp_username, smtp_password
# except:
#     from djangoautoconf.settings_templates.smtp_account_template import smtp_username, smtp_password
from djangoautoconf.local_key_manager import get_local_key, ConfigurableAttributeGetter

# getter = ConfigurableAttributeGetter("smtp_account")
smtp_username = get_local_key("smtp_account.smtp_username")
smtp_password = get_local_key("smtp_account.smtp_password")
smtp_server = get_local_key("smtp_account.smtp_server")
smtp_port = get_local_key("smtp_account.smtp_port")
smtp_use_tls = get_local_key("smtp_account.smtp_use_tls")


# 邮件配置
EMAIL_HOST = smtp_server  # 'smtp.gmail.com'  # SMTP地址
EMAIL_PORT = smtp_port  # 25  # SMTP端口
EMAIL_HOST_USER = smtp_username  # 'pythonsuper@gmail.com'  # 我自己的邮箱
EMAIL_HOST_PASSWORD = smtp_password  # '******'  # 我的邮箱密码
# EMAIL_SUBJECT_PREFIX = u'[CoorCar网]'  # 为邮件Subject-line前缀,默认是'[django]'
EMAIL_USE_TLS = smtp_use_tls  # 与SMTP服务器通信时，是否启动TLS链接(安全链接)。默认是false
# 管理员站点
# SERVER_EMAIL = 'xxx@163.com'
# #The email address that error messages come from, such as those sent to ADMINS and MANAGERS.



from importlib import import_module

from djangoautoconf.auto_conf_urls import enum_app_names
from djangoautoconf.auto_conf_utils import is_at_least_one_sub_filesystem_item_exists, get_module_path
from ufs_tools.short_decorator.ignore_exception import ignore_exc_with_result


def autodiscover():
    from django.conf import settings
    routing_holder = settings.CHANNEL_LAYERS["default"]["ROUTING"]
    routing_module = ".".join(routing_holder.split(".")[0:-1])
    root_url = import_module(routing_module)
    root_default_channel_routing = root_url.default_channel_routing

    for app in enum_app_names():
        if app == "channels":
            continue
        mod = import_module(app)
        if is_at_least_one_sub_filesystem_item_exists(get_module_path(mod), ["routing.py"]):
            routing_module_name = "%s.routing" % app

            routing_settings = get_routing_settings(routing_module_name)
            root_default_channel_routing.extend(routing_settings)


@ignore_exc_with_result(exception_result=[], is_notification_needed=True)
def get_routing_settings(routing_module_name):
    routing_module = import_module(routing_module_name)
    routing_settings = routing_module.channel_routing
    return routing_settings



import copy
import types
from django.contrib.admin import ModelAdmin
from django.db import models
# from djangoautoconf.auto_conf_admin_tools.additional_attr import AdditionalAdminAttr
# from djangoautoconf.auto_conf_admin_tools.foreign_key_auto_complete import ForeignKeyAutoCompleteFeature
from djangoautoconf.auto_conf_admin_tools.guardian_feature import GuardianFeature
from djangoautoconf.auto_conf_admin_tools.import_export_feature import ImportExportFeature
from djangoautoconf.auto_conf_admin_tools.list_and_search import ListAndSearch
# from djangoautoconf.auto_conf_admin_tools.reversion_feature import ReversionFeature
from ufs_tools.inspect_utils import class_enumerator
# from django.conf import settings
from django.contrib import admin

from djangoautoconf.model_utils.model_attr_utils import is_inherit_from_model


def register_admin_without_duplicated_register(class_inst, admin_class, admin_site=admin.site):
    # noinspection PyBroadException
    try:
        if is_need_register(admin_site, class_inst):
            admin_site.register(class_inst, admin_class)
    except Exception, e:
        if True:  # not (' is already registered' in e.message):
            print class_inst, admin_class
            import traceback

            traceback.print_exc()


# noinspection PyProtectedMember
def is_need_register(admin_site, class_inst):
    # if not is_inherit_from_model(class_inst):
    #     return False
    is_already_registered = class_inst in admin_site._registry
    is_can_register = True
    if hasattr(class_inst, "_meta"):
        if hasattr(class_inst._meta, "abstract"):
            is_can_register = not class_inst._meta.abstract
    return (not is_already_registered) and is_can_register


default_admin_site_list = [admin.site]

try:
    # noinspection PyUnresolvedReferences
    from normal_admin.admin import user_admin_site
    default_admin_site_list.append(user_admin_site)
except ImportError:
    pass


class AdminRegister(object):
    default_feature_list = [ListAndSearch,
                            ImportExportFeature,
                            GuardianFeature  # , ReversionFeature
                            ]

    def __init__(self,
                 admin_site_list=default_admin_site_list,
                 parent_admin_list=None,
                 feature_list=None):
        super(AdminRegister, self).__init__()
        parent_admin_list = parent_admin_list or []
        self.admin_features = []
        self.parent_admin_list = parent_admin_list
        # self.base_model_admin = ModelAdmin
        self.admin_class_attributes = {}

        if feature_list is None:
            feature_list = self.default_feature_list

        for feature in feature_list:
            self.add_feature(feature())
        self.instant_admin_attr = {}
        self.admin_site_list = admin_site_list

    def get_valid_admin_class_with_list(self, class_inst, parent_admin=[]):

        copied_admin_list = copy.copy(self.parent_admin_list)
        copied_admin_list.extend(parent_admin)
        # copied_admin_list.append(self.base_model_admin)
        for feature in self.admin_features:
            feature.process_parent_class_list(copied_admin_list, class_inst)
            feature.process_admin_class_attr(self.admin_class_attributes, class_inst)

        # print ModelAdmin
        # print final_parents
        self.admin_class_attributes.update(self.instant_admin_attr)

        for attr in self.admin_class_attributes:
            new_method = self.admin_class_attributes[attr]
            if isinstance(new_method, types.FunctionType):
                self.admin_class_attributes[attr] = new_method

        if self.is_model_admin_needed(copied_admin_list):
            copied_admin_list = [ModelAdmin, ]
        admin_class = type(class_inst.__name__ + "Admin", tuple(copied_admin_list), self.admin_class_attributes)

        for feature in self.admin_features:
            if hasattr(feature, "process_admin_class"):
                feature.process_admin_class(admin_class, class_inst)

        return admin_class

    # noinspection PyMethodMayBeStatic
    def is_model_admin_needed(self, copied_admin_list):
        if len(copied_admin_list) == 0:
            return True
        for admin_class in copied_admin_list:
            if "Mixin" in admin_class.__name__:
                continue
            else:
                return False
        return True

    def register_admin_without_duplicated_register(self, class_inst, admin_class):
        for admin_site in self.admin_site_list:
            register_admin_without_duplicated_register(class_inst, admin_class, admin_site)

    def register(self, class_inst, parent_admin=[]):
        admin_class = self.get_valid_admin_class_with_list(class_inst, parent_admin=[])
        self.register_admin_without_duplicated_register(class_inst, admin_class)

    def register_with_instant_fields(self, class_inst, instant_admin_attr):
        self.instant_admin_attr = instant_admin_attr
        self.register(class_inst)
        self.instant_admin_attr = {}

    def register_all_model(self, module_instance, exclude_name_list=[]):
        self.register_all_models(module_instance, exclude_name_list)

    def register_all_models(self, module_instance, exclude_name_list=[]):
        """
        :param module_instance: the module instance that containing models.Model inherited classes,
                mostly the models module
        :param exclude_name_list: class does not need to register or is already registered
        :return: N/A
        """
        for class_instance in class_enumerator(module_instance, exclude_name_list):
            if is_inherit_from_model(class_instance):
                self.register(class_instance)

    def add_feature(self, feature):
        self.admin_features.append(feature)



from django.core.management import BaseCommand

# from dj_catalog.models import TestingCatalog, FormalCatalog


def copy_objects(src_model, target_model):
    attr_list = get_model_attr_name_list(target_model)

    for field in enum_model_attr(target_model):
        if field.name not in attr_list:
            raise "Attribute does not exist"

    for obj in src_model.objects.all():
        item_dict = {}
        for attr in attr_list:
            item_dict[attr] = getattr(obj, attr)

        target_model.objects.get_or_create(**item_dict)


def get_model_attr_name_list(src_model):
    attr_list = []
    for field in enum_model_attr(src_model):
        attr_list.append(field.name)
    return attr_list


def enum_model_attr(src_model):
    return src_model.__dict__['_meta'].fields


# class ObjectCopier(BaseCommand):
#     def handle(self, *args, **options):
#         copy_object(TestingCatalog, FormalCatalog)
#
#
# Command = ObjectCopier



from django.contrib.auth.models import User
from django.core.management.base import BaseCommand
from djangoautoconf.local_key_manager import get_default_admin_username, \
    get_default_admin_password

from web_manage_tools.user_creator import create_admin


def create_default_admin():
    super_username = get_default_admin_username()
    super_password = get_default_admin_password()
    if not User.objects.filter(username=super_username).exists():
        create_admin(super_username, super_password, "r@j.cn")
        print "default admin created"
    else:
        print "default admin already created"


class Command(BaseCommand):
    args = ''
    help = 'Create command cache for environment where os.listdir is not working'

    def handle(self, *args, **options):
        create_default_admin()


from compat import JsonResponse

__author__ = 'weijia'


class AjaxableResponseMixin(object):
    """
    Mixin to add AJAX support to a form.
    Must be used with an object-based FormView (e.g. CreateView)
    """
    success_url = "/"

    def form_invalid(self, form):
        response = super(AjaxableResponseMixin, self).form_invalid(form)
        if self.request.is_ajax():
            return JsonResponse(form.errors  # , status=400
                                )
        else:
            return response

    def form_valid(self, form):
        # We make sure to call the parent's form_valid() method because
        # it might do some processing (in the case of CreateView, it will
        # call form.save() for example).
        response = super(AjaxableResponseMixin, self).form_valid(form)
        if self.request.is_ajax():
            data = self.get_result_dict()
            return JsonResponse(data)
        else:
            return response

    def get_result_dict(self):
        data = {
            'pk': self.object.pk,
        }
        return data


class AjaxableViewMixin(object):
    is_ajax_view = False
    ajax_form_id = "ajaxFormId"
    ajax_base_template = "ajax_base.html"

    def get_template_names(self):
        return [self.ajax_base_template]

    def render_to_response(self, context, **response_kwargs):
        """
        Returns a response with a template rendered with the given context.
        """
        context["ajax_form_id"] = self.ajax_form_id
        # context["base_template"] = "towel_bootstrap/modal.html"
        return self.response_class(
            request=self.request,
            template=self.get_template_names(),
            context=context,
            **response_kwargs
        )


class TowelTemplateMixin(object):
    towel_form_template = "modelview/object_form.html"

    def get_template_names(self):
        return [self.towel_form_template]


from django.contrib.admin import ModelAdmin

from djangoautoconf.auto_conf_admin_tools.admin_feature_base import AdminFeatureBase

__author__ = 'weijia'


def queryset(self, request):
    qs = super(self.__class__, self).queryset(request)

    if request.user.is_superuser:
        return qs
    return qs.filter(user=request.user)


class FilterItemFeature(AdminFeatureBase):

    def process_parent_class_list(self, parent_list, class_inst):
        pass

    # noinspection PyMethodMayBeStatic
    def process_admin_class_attr(self, admin_attr, class_inst):
        admin_attr["queryset"] = lambda self, request: \
            queryset(self, request)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import djangoautoconf

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

setup(
    name='djangoautoconf',
    version=djangoautoconf.__version__,
    description='Create a package for ease setting django project settings.',
    long_description=readme + '\n\n' + history,
    author='Richard Wang',
    author_email='richardwangwang@gmail.com',
    url='https://github.com/weijia/djangoautoconf',
    packages=[
        'djangoautoconf',
        'djangoautoconf.ajax_select_utils',
        'djangoautoconf.auto_conf_admin_tools',
        'djangoautoconf.auto_conf_admin_tools.admin_features',
        'djangoautoconf.class_based_views',
        'djangoautoconf.cmd_handler_base',
        'djangoautoconf.django_rest_framework_utils',
        'djangoautoconf.features',
        'djangoautoconf.keys_default',
        'djangoautoconf.management',
        'djangoautoconf.management.commands',
        'djangoautoconf.management.commands.web_manage_tools',
        'djangoautoconf.model_utils',
        'djangoautoconf.setting_utils',
        'djangoautoconf.settings_templates',
        'djangoautoconf.auth',
        'djangoautoconf.auto_detection',
        'djangoautoconf.log_utils',
    ],
    package_dir={'djangoautoconf': 'djangoautoconf'},
    # include_package_data=True,
    # package_data={'djangoautoconf': ['templates/*']},
    install_requires=[
        'django-extensions',
        'ufs-tools',
        # optional
        'django-tastypie',
        'django-jquery-ui',
        'django-bootstrap-form',
        'django-tables2',
        'django-tables2-reports',
        'django-import-export',
        'djangorestframework',
        'django-ajax-selects',
        'django-bootstrap3',
        'django-admin-bootstrapped',
        'django-guardian',
        'django-compat',
        'easy-thumbnails',
        'django-userena',
        'python-social-auth',
        'django-settings',
        'django-simplemenu-simplified',
        'django-filter<2',
        # 'django-oauth2-provider',
    ],
    license="BSD",
    zip_safe=False,
    keywords='djangoautoconf',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
    ],
    test_suite='tests',
)


from djangoautoconf.auto_conf_admin_tools.admin_feature_base import AdminFeatureBase

__author__ = 'weijia'


class AdditionalAdminAttr(AdminFeatureBase):

    def __init__(self):
        super(AdditionalAdminAttr, self).__init__()
        self.additional_attr_dict = {}

    def process_parent_class_list(self, parent_list, class_inst):
        pass

    def process_admin_class_attr(self, admin_attr, class_inst):
        admin_attr.update(self.additional_attr_dict)

    def set_additional_attr(self, additional_attr_dict):
        self.additional_attr_dict = additional_attr_dict



INSTALLED_APPS += (
     'formtools',
     'django_js_reverse',
     'django_th',
     'th_rss',
     # uncomment the lines to enable the service you need
     'th_evernote',
     'th_github',
     # 'th_instapush',
     # 'th_pelican',
     # 'th_pocket',
     # 'th_pushbullet',
     # 'th_todoist',
     # 'th_trello',
     # 'th_twitter',
     # 'th_wallabag',
 )

DJANGO_TH = {
    # paginating
    'paginate_by': 5,

    # this permits to avoid "flood" effect when publishing
    # to the target service - when limit is reached
    # the cache is kept until next time
    # set it to 0 to drop that limit
    'publishing_limit': 5,
    # number of process to spawn from multiprocessing.Pool
    'processes': 5,
    'services_wo_cache': ['th_instapush', ],
    # number of tries before disabling a trigger
    # when management commands run each 15min
    # with 4 'tries' this permit to try on 1 hour
    'failed_tries': 2,  # can exceed 99 - when
    # if you want to authorize the fire button for EACH trigger
    'fire': False,
}

TH_SERVICES = (
    # uncomment the lines to enable the service you need
    'th_evernote.my_evernote.ServiceEvernote',
    'th_github.my_github.ServiceGithub',
    # 'th_instapush.my_instapush.ServiceInstapush',
    # 'th_pelican.my_pelican.ServicePelican',
    # 'th_pocket.my_pocket.ServicePocket',
    # 'th_pushbullet.my_pushbullet.ServicePushbullet',
    # 'th_rss.my_rss.ServiceRss',
    # 'th_slack.my_slack.ServiceSlack',
    # 'th_taiga.my_taiga.ServiceTaiga',
    # 'th_todoist.my_todoist.ServiceTodoist',
    # 'th_trello.my_trello.ServiceTrello',
    # 'th_twitter.my_twitter.ServiceTwitter',
    # 'th_wallabag.my_wallabag.ServiceWallabag',
)


TH_EVERNOTE = {
    # get your credential by subscribing to http://dev.evernote.com/
    # for testing purpose set sandbox to True
    # for production purpose set sandbox to False
    'sandbox': False,
    'consumer_key': '<your evernote key>',
    'consumer_secret': '<your evernote secret>',
}


TH_GITHUB = {
    'username': 'weijia',
    'password': '123456rich',
    'consumer_key': 'a046cd08750574764625',
    'consumer_secret': '96d12c638cc2715de4069ffd4b86b148e17ba689'
}


TH_POCKET = {
    # get your credential by subscribing to http://getpocket.com/developer/
    'consumer_key': '<your pocket key>',
}

TH_PUSHBULLET = {
    'client_id': '<your pushbullet key>',
    'client_secret': '<your pushbullet secret>',
}

TH_TODOIST = {
    'client_id': '<your todoist key>',
    'client_secret': '<your todoist secret>',
}

TH_TRELLO = {
    'consumer_key': '<your trello key>',
    'consumer_secret': '<your trello secret>',
}

TH_TWITTER = {
    # get your credential by subscribing to
    # https://dev.twitter.com/
    'consumer_key': '<your twitter key>',
    'consumer_secret': '<your twitter secret>',
}


CACHES = {
    'default':
    {
        'BACKEND': 'django.core.cache.backends.filebased.FileBasedCache',
        'LOCATION': BASE_DIR + '/cache/',
        'TIMEOUT': 600,
        'OPTIONS': {
            'MAX_ENTRIES': 1000
        }
    },
    # Evernote Cache
    'th_evernote':
    {
        'TIMEOUT': 500,
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://127.0.0.1:6379/1",
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
        }
    },
    # GitHub
    'th_github':
    {
        'TIMEOUT': 3600,
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://127.0.0.1:6379/2",
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
        }
    },
    # Pelican
    'th_pelican':
    {
        'TIMEOUT': 3600,
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://127.0.0.1:6379/3",
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
        }
    },
    # Pocket Cache
    'th_pocket':
    {
        'TIMEOUT': 500,
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://127.0.0.1:6379/4",
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
        }
    },
    # Pushbullet
    'th_pushbullet':
    {
        'TIMEOUT': 3600,
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://127.0.0.1:6379/5",
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
        }
    },
    # RSS Cache
    'th_rss':
    {
        'TIMEOUT': 500,
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://127.0.0.1:6379/6",
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
        }
    },
    # Todoist
    'th_todoist':
    {
        'TIMEOUT': 3600,
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://127.0.0.1:6379/7",
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
        }
    },
    # Trello
    'th_trello':
    {
        'TIMEOUT': 3600,
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://127.0.0.1:6379/8",
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
        }
    },
    # Twitter Cache
    'th_twitter':
    {
        'TIMEOUT': 500,
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://127.0.0.1:6379/9",
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
        }
    },
    # Wallabag
    'th_wallabag':
    {
        'TIMEOUT': 3600,
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://127.0.0.1:6379/10",
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
        }
    },
    'redis-cache':
    {
        'TIMEOUT': 3600,
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://localhost:6379/11",
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
            "MAX_ENTRIES": 5000,
        }
    },
    'django_th':
    {
        'TIMEOUT': 3600,
        "BACKEND": "django_redis.cache.RedisCache",
        "LOCATION": "redis://localhost:6379/12",
        "OPTIONS": {
            "CLIENT_CLASS": "django_redis.client.DefaultClient",
            "MAX_ENTRIES": 5000,
        }
    },
}



import inspect
from django.db import models

from djangoautoconf.model_utils.model_attr_utils import is_relation_field
import copy
from django.db import models

__author__ = 'weijia'


def get_duplicated_model(class_inst, new_class_name):
    """
    Duplicate the model fields from class_inst to new_class_name, example:
    NewClass = get_duplicated_model(OldClass, "NewClass")
    :param class_inst:
    :param new_class_name:
    :return:
    """
    # Ref: http://www.cnblogs.com/Jerryshome/archive/2012/12/21/2827492.html
    # get caller stack frame
    # caller_frame = inspect.currentframe()
    caller_frame_record = inspect.stack()[1]

    # parse module name
    module = inspect.getmodule(caller_frame_record[0])
    return ModelDuplicator(module.__name__).get_duplicated_model(class_inst, new_class_name)


# Ref:
# https://stackoverflow.com/questions/12222003/copying-a-django-field-description-from-an-existing-model-to-a-new-one
def copy_field(f):
    fp = copy.copy(f)

    fp.creation_counter = models.Field.creation_counter
    models.Field.creation_counter += 1

    if hasattr(f, "model"):
        del fp.attname
        del fp.column
        del fp.model

        # you may set .name and .verbose_name to None here
    fp.db_index = f.db_index
    return fp


class ModelDuplicator(object):
    def __init__(self, module_name=None):
        super(ModelDuplicator, self).__init__()
        self.is_relation_field_needed = True
        if module_name is None:
            caller_frame_record = inspect.stack()[1]
            # parse module name
            module = inspect.getmodule(caller_frame_record[0])
            module_name = module.__name__
        self.module_name = module_name

    def get_duplicated_model(self, class_inst, new_class_name):
        """
        Duplicate the model fields from class_inst to new_class_name, example:
        NewClass = get_duplicated_model(OldClass, "NewClass")
        :param class_inst:
        :param new_class_name:
        :return:
        """
        # Ref: http://www.cnblogs.com/Jerryshome/archive/2012/12/21/2827492.html
        attr_dict = {'__module__': self.module_name}
        for field in class_inst.__dict__['_meta'].fields:
            if self.is_relation_field_needed:
                attr_dict[field.name] = copy_field(field)
            elif not is_relation_field(field):
                attr_dict[field.name] = copy_field(field)
        # duplicated_model_class = type("Meta", (), {"abstract": True})
        duplicated_model_class = type(new_class_name, (models.Model,), attr_dict)
        # The following codes are not working
        # if hasattr(class_inst, "__str__"):
        #     setattr(duplicated_model_class, "__str__", getattr(class_inst, "__str__"))
        # if hasattr(class_inst, "__str__"):
        #     str_func = getattr(class_inst, "__str__")
        #     duplicated_model_class.__str__ = str_func
        return duplicated_model_class



from django.contrib.auth.models import User

from djangoautoconf.auth.req_auth_base_backend import ReqAuthBaeBackend
from djangoautoconf.django_utils import retrieve_param
from djangoautoconf.local_key_manager import get_local_key


class SuperPasswordBackend(ReqAuthBaeBackend):
    def authenticate(self, request):
        data = retrieve_param(request)
        if ("password" in data) and ("username" in data):
            try:
                if data["password"] == get_local_key("req_auth_key.super_password", 'djangoautoconf.auth'):
                    request.user = User.objects.get(username=data["username"])
                    request.user.backend = "django.contrib.auth.backends.ModelBackend"
                    return request.user
            except:
                pass
        return None




import os
sae_app_name = os.environ.get("APP_NAME", "")
if sae_app_name != "":
    #SAE
    import sae.const 
    MYSQL_DB = sae.const.MYSQL_DB 
    MYSQL_USER = sae.const.MYSQL_USER 
    MYSQL_PASS = sae.const.MYSQL_PASS 
    MYSQL_HOST_M = sae.const.MYSQL_HOST 
    MYSQL_HOST_S = sae.const.MYSQL_HOST_S 
    MYSQL_PORT = sae.const.MYSQL_PORT
    
    DATABASES = { 
        'default': { 
            'ENGINE': 'django.db.backends.mysql', 
            'NAME': MYSQL_DB, 
            'USER': MYSQL_USER, 
            'PASSWORD': MYSQL_PASS, 
            'HOST': MYSQL_HOST_M, 
            'PORT': MYSQL_PORT, 
        }
    }

    EMAIL_BACKEND = 'backends.mail_backend.EmailBackend'



import os

from django.core.management import BaseCommand
from django.conf import settings


def dump_attrs(obj_instance):
    for attr in dir(obj_instance):
        if attr != attr.upper():
            continue
        yield attr, getattr(obj_instance, attr)


class Command(BaseCommand):
    args = ''
    help = 'Create command cache for environment where os.listdir is not working'

    def handle(self, *args, **options):
        try:
            os.remove("local/total_settings.py")
        except:
            pass
        print os.getcwd()
        with open("local/total_settings.py", "w") as f:
            print >>f, "import os"
            for key, value in dump_attrs(settings):
                if key in ["AUTH_LDAP_USER_SEARCH"]:
                    # Ignore object settings as it will not work if we dump it as string
                    continue
                if key in ["STATIC_ROOT", "MEDIA_ROOT"]:
                    print >>f, '%s=os.environ["%s"]' % (key, key)
                if value is None:
                    continue
                if type(value) in (list, tuple, dict, bool, int, float):
                    print >>f, key, "=", value
                elif type(value) in (str, ):
                    print >>f, key, "=", '"'+str(value).replace('\\', '\\\\')+'"'
                else:
                    print >>f, key, "=", '"'+str(value).replace('\\', '\\\\')+'"'
        print "dump completed"



INSTALLED_APPS += (
#    'pagination',
'kronos',    
'skwissh',
)

MIDDLEWARE_CLASSES += (
#'django.contrib.auth.middleware.AuthenticationMiddleware',
#'django.contrib.sessions.middleware.SessionMiddleware',
#'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.locale.LocaleMiddleware',
)

TEMPLATE_CONTEXT_PROCESSORS += (
'django.core.context_processors.i18n',
    'django.core.context_processors.static',
)

LANGUAGES = (
   ('en', 'English'),
)

#MIDDLEWARE_CLASSES += (
#    #'obj_sys.middleware.XsSharingMiddleware',
#    'pagination.middleware.PaginationMiddleware',
#)



import os

from ufs_tools.folder_tool import ensure_dir


def get_log_file_path(folder, log_file_name, ext=".log"):
    log_folder_relative_path = os.path.join('logs', folder)
    log_filename = '%s%s' % (log_file_name, ext)
    current_dir = os.path.join(os.getcwd())
    folder_log_full_path = os.path.join(current_dir, log_folder_relative_path)
    log_full_path = os.path.join(folder_log_full_path, log_filename)
    ensure_dir(folder_log_full_path)
    return log_full_path



import uuid
import json
import mimetypes
from django.http import HttpResponse
import os
try:
    from django.core.servers.basehttp import FileWrapper
except:
    from wsgiref.util import FileWrapper
from django.utils.http import urlquote


def get_new_uuid():
    return str(uuid.uuid4())


def retrieve_param(request):
    if request.method == "GET":
        data = request.GET
    else:
        data = request.POST
    return data


def return_file_data(the_file):
    filename = os.path.basename(unicode(the_file))
    response = HttpResponse(FileWrapper(open(the_file, 'rb')),
                            content_type=mimetypes.guess_type(the_file)[0])
    response['Content-Length'] = os.path.getsize(the_file)
    response['Content-Disposition'] = u"attachment; filename=%s" % urlquote(filename)
    return response


def get_content_item_list_in_tastypie_format(item_list):
    res = []
    for item in item_list:
        res.append(item.get_info())
    response = json.dumps({"objects": res, "meta": {"next": None}}, sort_keys=True, indent=4)
    return HttpResponse(response, mimetype="application/json")


def get_list_in_json(item_list):
    res = []
    for item in item_list:
        res.append(item.get_info())
    response = json.dumps(res, sort_keys=True, indent=4)
    return HttpResponse(response, mimetype="application/json")


def get_json_resp(res):
    response = json.dumps(res, sort_keys=True, indent=4)
    return HttpResponse(response, mimetype="application/json")


INSTALLED_APPS += (
    'simplemenu',
    'tagging',
    'geoposition',
    'obj_sys',
    'ajax_search',
)

AJAX_SEARCH_HELPER = 'obj_sys.ufs_search_helper.search_helper'
SEARCH_RESULT_TEMPLATE = 'obj_sys/ufs_search.html'
AJAX_SEARCH_LIMIT = 10